@@ -16,6 +16,7 @@
 #include <sys/resource.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
+#include <sys/statfs.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -29,6 +30,7 @@
 #endif
 
 #define KRUN_EXIT_CODE_IOCTL 0x7602
+#define KRUN_REMOVE_ROOT_DIR_IOCTL 0x7603
 
 #define KRUN_MAGIC "KRUN"
 #define KRUN_FOOTER_LEN 12
@@ -956,10 +958,35 @@ int setup_redirects()
     return 0;
 }
 
+int is_virtiofs(const char *path)
+{
+    struct statfs fs;
+
+    if (statfs(path, &fs) != 0) {
+        perror("statfs");
+        return -1;
+    }
+
+    // virtiofs magic number: 0x65735546
+    return (fs.f_type == 0x65735546) ? 1 : 0;
+}
+
 void set_exit_code(int code)
 {
     int fd;
     int ret;
+    int virtiofs_check;
+
+    // Only use the ioctl if virtiofs is used for root filesystem
+    virtiofs_check = is_virtiofs("/");
+    if (virtiofs_check < 0) {
+        printf("Warning: Could not determine filesystem type for root\n");
+    }
+
+    if (virtiofs_check == 0) {
+        // Root filesystem is not virtiofs, skip the ioctl
+        return;
+    }
 
     fd = open("/", O_RDONLY);
     if (fd < 0) {
@@ -975,9 +1002,45 @@ void set_exit_code(int code)
     close(fd);
 }
 
+int try_mount(const char *source, const char *target, const char *fstype,
+              unsigned long mountflags, const void *data)
+{
+    FILE *f;
+    char line[129];
+    int mount_status = -1;
+
+    if (fstype) {
+        return mount(source, target, fstype, mountflags, data);
+    }
+
+    f = fopen("/proc/filesystems", "r");
+    if (f == NULL) {
+        perror("fopen(/proc/filesystems)");
+        return -1;
+    }
+    while (fgets(line, sizeof(line), f)) {
+        char fstype[sizeof(line)];
+        if (!strncmp(line, "nodev", 5)) {
+            continue;
+        }
+        if (sscanf(line, "%128s\n", fstype) != 1) {
+            continue;
+        }
+
+        mount_status = mount(source, target, fstype, mountflags, data);
+        if (mount_status == 0) {
+            break;
+        }
+    }
+    fclose(f);
+
+    return mount_status;
+}
+
 int main(int argc, char **argv)
 {
     struct ifreq ifr;
+    int fd;
     int sockfd;
     int status;
     int saved_errno;
@@ -986,10 +1049,35 @@ int main(int argc, char **argv)
     char *krun_home;
     char *krun_term;
     char *krun_init;
+    char *krun_root;
+    char *krun_root_fstype;
+    char *krun_root_options;
     char *config_workdir, *env_workdir;
     char *rlimits;
     char **config_argv, **exec_argv;
 
+#ifdef TDX
+    if (mkdir("/tmp", 0755) < 0 && errno != EEXIST) {
+        perror("mkdir(/tmp)");
+        exit(-1);
+    }
+    if (mkdir("/tmp/vda", 0755) < 0 && errno != EEXIST) {
+        perror("mkdir(/tmp/vda)");
+        exit(-1);
+    }
+    if (mount("/dev/vda", "/tmp/vda", "ext4", MS_RELATIME, NULL) < 0) {
+        perror("mount(/dev/vda)");
+        exit(-1);
+    }
+    chdir("/tmp/vda");
+    if (mount(".", "/", NULL, MS_MOVE, NULL) < 0) {
+        perror("remount root");
+        exit(-1);
+    }
+    chroot(".");
+
+#endif
+
 #ifdef SEV
     if (chroot_luks() < 0) {
         printf("Couldn't switch to LUKS volume, bailing out\n");
@@ -1001,6 +1089,47 @@ int main(int argc, char **argv)
         exit(-2);
     }
 
+    krun_root = getenv("KRUN_BLOCK_ROOT_DEVICE");
+    if (krun_root) {
+        if (mkdir("/newroot", 0755) < 0 && errno != EEXIST) {
+            perror("mkdir(/newroot)");
+            exit(-1);
+        }
+
+        krun_root_fstype = getenv("KRUN_BLOCK_ROOT_FSTYPE");
+        krun_root_options = getenv("KRUN_BLOCK_ROOT_OPTIONS");
+
+        if (try_mount(krun_root, "/newroot", krun_root_fstype, 0,
+                      krun_root_options) < 0) {
+            perror("mount KRUN_BLOCK_ROOT_DEVICE");
+            exit(-1);
+        }
+
+        chdir("/newroot");
+
+        fd = open("/", O_RDONLY);
+        if (fd < 0) {
+            perror("Couldn't open temporary root directory for removing");
+            exit(-1);
+        }
+        if (ioctl(fd, KRUN_REMOVE_ROOT_DIR_IOCTL) < 0) {
+            perror("Error removing temporary root directory");
+        }
+        close(fd);
+
+        if (mount(".", "/", NULL, MS_MOVE, NULL) < 0) {
+            perror("remount root");
+            exit(-1);
+        }
+        chroot(".");
+
+        // we must mount filesystems again after chrooting
+        if (mount_filesystems() < 0) {
+            printf("Couldn't mount filesystems, bailing out\n");
+            exit(-2);
+        }
+    }
+
     setsid();
     ioctl(0, TIOCSCTTY, 1);
 
