@@ -17,12 +17,13 @@ pub mod msr;
 /// Logic for configuring x86_64 registers.
 pub mod regs;
 
-use crate::{round_up, ArchMemoryInfo, InitrdConfig};
+use crate::{ArchMemoryInfo, InitrdConfig};
 use arch_gen::x86::bootparam::{boot_params, E820_RAM};
 use vm_memory::Bytes;
 use vm_memory::{
     Address, ByteValued, GuestAddress, GuestMemory, GuestMemoryMmap, GuestMemoryRegion,
 };
+use vmm_sys_util::align_upwards;
 
 // This is a workaround to the Rust enforcement specifying that any implementation of a foreign
 // trait (in this case `ByteValued`) where:
@@ -51,7 +52,10 @@ pub enum Error {
 
 // Where BIOS/VGA magic would live on a real PC.
 const EBDA_START: u64 = 0x9fc00;
+#[cfg(not(feature = "tdx"))]
 pub const RESET_VECTOR: u64 = 0xfff0;
+#[cfg(feature = "tdx")]
+pub const RESET_VECTOR: u64 = 0xffff_fff0;
 pub const RESET_VECTOR_SEV_AP: u64 = 0xfff3;
 pub const BIOS_START: u64 = 0xffff_0000;
 pub const BIOS_SIZE: usize = 65536;
@@ -73,7 +77,7 @@ pub fn arch_memory_regions(
 ) -> (ArchMemoryInfo, Vec<(GuestAddress, usize)>) {
     let page_size: usize = unsafe { libc::sysconf(libc::_SC_PAGESIZE).try_into().unwrap() };
 
-    let size = round_up(size, page_size);
+    let size = align_upwards!(size, page_size);
 
     // It's safe to cast MMIO_MEM_START to usize because it fits in a u32 variable
     // (It points to an address in the 32 bit space).
@@ -155,7 +159,7 @@ pub fn arch_memory_regions(
 ) -> (ArchMemoryInfo, Vec<(GuestAddress, usize)>) {
     let page_size: usize = unsafe { libc::sysconf(libc::_SC_PAGESIZE).try_into().unwrap() };
 
-    let size = round_up(size, page_size);
+    let size = align_upwards!(size, page_size);
     if let Some(kernel_load_addr) = kernel_load_addr {
         if size < (kernel_load_addr + kernel_size as u64) as usize {
             panic!("Kernel doesn't fit in RAM");
@@ -268,12 +272,19 @@ pub fn configure_system(
         params.0.hdr.ramdisk_size = initrd_config.size as u32;
     }
 
-    #[cfg(feature = "tee")]
-    {
-        params.0.hdr.syssize = num_cpus as u32;
-    }
+    if cfg!(feature = "tdx") {
+        // number of 4k pages
+        params.0.hdr.syssize = (arch_memory_info.ram_last_addr / 4096) as u32;
+        // nuymber of vCPUs
+        params.0.hdr.root_flags = num_cpus as u16;
 
-    add_e820_entry(&mut params.0, 0, EBDA_START, E820_RAM)?;
+        add_e820_entry(&mut params.0, 0, EBDA_START - 0x10000, E820_RAM)?;
+    } else {
+        if cfg!(feature = "tee") {
+            params.0.hdr.syssize = num_cpus as u32;
+        }
+        add_e820_entry(&mut params.0, 0, EBDA_START, E820_RAM)?;
+    }
 
     let last_addr = GuestAddress(arch_memory_info.ram_last_addr);
     if last_addr < end_32bit_gap_start {
