@@ -1,15 +1,12 @@
-use crate::legacy::IrqChip;
 use crate::virtio::descriptor_utils::{Reader, Writer};
-use crate::Error as DeviceError;
 
-use super::super::{Queue, VIRTIO_MMIO_INT_VRING};
+use super::super::Queue;
 use super::device::{CacheType, DiskProperties};
 
+use crate::virtio::InterruptTransport;
 use std::io::{self, Write};
 use std::os::fd::AsRawFd;
 use std::result;
-use std::sync::atomic::{AtomicUsize, Ordering};
-use std::sync::Arc;
 use std::thread;
 use utils::epoll::{ControlOperation, Epoll, EpollEvent, EventSet};
 use utils::eventfd::EventFd;
@@ -49,11 +46,7 @@ unsafe impl ByteValued for RequestHeader {}
 pub struct BlockWorker {
     queue: Queue,
     queue_evt: EventFd,
-    interrupt_status: Arc<AtomicUsize>,
-    interrupt_evt: EventFd,
-    intc: Option<IrqChip>,
-    irq_line: Option<u32>,
-
+    interrupt: InterruptTransport,
     mem: GuestMemoryMmap,
     disk: DiskProperties,
     stop_fd: EventFd,
@@ -64,10 +57,7 @@ impl BlockWorker {
     pub fn new(
         queue: Queue,
         queue_evt: EventFd,
-        interrupt_status: Arc<AtomicUsize>,
-        interrupt_evt: EventFd,
-        intc: Option<IrqChip>,
-        irq_line: Option<u32>,
+        interrupt: InterruptTransport,
         mem: GuestMemoryMmap,
         disk: DiskProperties,
         stop_fd: EventFd,
@@ -75,11 +65,7 @@ impl BlockWorker {
         Self {
             queue,
             queue_evt,
-            interrupt_status,
-            interrupt_evt,
-            intc,
-            irq_line,
-
+            interrupt,
             mem,
             disk,
             stop_fd,
@@ -129,16 +115,14 @@ impl BlockWorker {
                             }
                             _ => {
                                 log::warn!(
-                                    "Received unknown event: {:?} from fd: {:?}",
-                                    event_set,
-                                    source
+                                    "Received unknown event: {event_set:?} from fd: {source:?}"
                                 );
                             }
                         }
                     }
                 }
                 Err(e) => {
-                    debug!("failed to consume muxer epoll event: {}", e);
+                    debug!("failed to consume muxer epoll event: {e}");
                 }
             }
         }
@@ -146,7 +130,7 @@ impl BlockWorker {
 
     fn process_queue_event(&mut self) {
         if let Err(e) = self.queue_evt.read() {
-            error!("Failed to get queue event: {:?}", e);
+            error!("Failed to get queue event: {e:?}");
         } else {
             self.process_virtio_queues();
         }
@@ -171,21 +155,21 @@ impl BlockWorker {
             let mut reader = match Reader::new(mem, head.clone()) {
                 Ok(r) => r,
                 Err(e) => {
-                    error!("invalid descriptor chain: {:?}", e);
+                    error!("invalid descriptor chain: {e:?}");
                     continue;
                 }
             };
             let mut writer = match Writer::new(mem, head.clone()) {
                 Ok(r) => r,
                 Err(e) => {
-                    error!("invalid descriptor chain: {:?}", e);
+                    error!("invalid descriptor chain: {e:?}");
                     continue;
                 }
             };
             let request_header: RequestHeader = match reader.read_obj() {
                 Ok(h) => h,
                 Err(e) => {
-                    error!("invalid request header: {:?}", e);
+                    error!("invalid request header: {e:?}");
                     continue;
                 }
             };
@@ -194,22 +178,22 @@ impl BlockWorker {
                 match self.process_request(request_header, &mut reader, &mut writer) {
                     Ok(l) => (VIRTIO_BLK_S_OK.try_into().unwrap(), l),
                     Err(e) => {
-                        error!("error processing request: {:?}", e);
+                        error!("error processing request: {e:?}");
                         (VIRTIO_BLK_S_IOERR.try_into().unwrap(), 0)
                     }
                 };
 
             if let Err(e) = writer.write_obj(status) {
-                error!("Failed to write virtio block status: {:?}", e)
+                error!("Failed to write virtio block status: {e:?}")
             }
 
             if let Err(e) = self.queue.add_used(mem, head.index, len as u32) {
-                error!("failed to add used elements to the queue: {:?}", e);
+                error!("failed to add used elements to the queue: {e:?}");
             }
 
             if self.queue.needs_notification(mem).unwrap() {
-                if let Err(e) = self.signal_used_queue() {
-                    error!("error signalling queue: {:?}", e);
+                if let Err(e) = self.interrupt.try_signal_used_queue() {
+                    error!("error signalling queue: {e:?}");
                 }
             }
         }
@@ -266,15 +250,4 @@ impl BlockWorker {
             _ => Err(RequestError::UnknownRequest),
         }
     }
-
-    fn signal_used_queue(&self) -> result::Result<(), DeviceError> {
-        self.interrupt_status
-            .fetch_or(VIRTIO_MMIO_INT_VRING as usize, Ordering::SeqCst);
-        if let Some(intc) = &self.intc {
-            intc.lock()
-                .unwrap()
-                .set_irq(self.irq_line, Some(&self.interrupt_evt))?;
-        }
-        Ok(())
-    }
 }
