@@ -3,7 +3,6 @@ use std::io::Write;
 use std::iter::zip;
 use std::mem::{size_of, size_of_val};
 use std::os::unix::io::{AsRawFd, RawFd};
-use std::sync::atomic::AtomicUsize;
 use std::sync::Arc;
 
 use libc::TIOCGWINSZ;
@@ -15,17 +14,15 @@ use super::super::{
     ActivateError, ActivateResult, ConsoleError, DeviceState, Queue as VirtQueue, VirtioDevice,
 };
 use super::{defs, defs::control_event, defs::uapi};
-use crate::legacy::IrqChip;
 use crate::virtio::console::console_control::{
     ConsoleControl, VirtioConsoleControl, VirtioConsoleResize,
 };
 use crate::virtio::console::defs::QUEUE_SIZE;
-use crate::virtio::console::irq_signaler::IRQSignaler;
 use crate::virtio::console::port::Port;
 use crate::virtio::console::port_queue_mapping::{
     num_queues, port_id_to_queue_idx, QueueDirection,
 };
-use crate::virtio::{PortDescription, VmmExitObserver};
+use crate::virtio::{InterruptTransport, PortDescription, VmmExitObserver};
 
 pub(crate) const CONTROL_RXQ_INDEX: usize = 2;
 pub(crate) const CONTROL_TXQ_INDEX: usize = 3;
@@ -50,7 +47,12 @@ pub(crate) fn get_win_size() -> (u16, u16) {
     let ret = unsafe { tiocgwinsz(0, &mut ws) };
 
     if let Err(err) = ret {
-        error!("Couldn't get terminal dimensions: {}", err);
+        match err {
+            // If the port isn't a TTY, this is expected to fail. Avoid logging
+            // an error in that case.
+            nix::errno::Errno::ENOTTY => {}
+            _ => error!("Couldn't get terminal dimensions: {err}"),
+        }
         (0, 0)
     } else {
         (ws.cols, ws.rows)
@@ -82,7 +84,6 @@ impl VirtioConsoleConfig {
 
 pub struct Console {
     pub(crate) device_state: DeviceState,
-    pub(crate) irq: IRQSignaler,
     pub(crate) control: Arc<ConsoleControl>,
     pub(crate) ports: Vec<Port>,
 
@@ -122,7 +123,6 @@ impl Console {
             .collect();
 
         Ok(Console {
-            irq: IRQSignaler::new(),
             control: ConsoleControl::new(),
             ports,
             queues,
@@ -142,16 +142,12 @@ impl Console {
         defs::CONSOLE_DEV_ID
     }
 
-    pub fn set_intc(&mut self, intc: IrqChip) {
-        self.irq.set_intc(intc)
-    }
-
     pub fn get_sigwinch_fd(&self) -> RawFd {
         self.sigwinch_evt.as_raw_fd()
     }
 
     pub fn update_console_size(&mut self, cols: u16, rows: u16) {
-        log::debug!("update_console_size: {} {}", cols, rows);
+        log::debug!("update_console_size: {cols} {rows}");
         // Note that we currently only support resizing on the first/main console
         self.control
             .console_resize(0, VirtioConsoleResize { rows, cols });
@@ -159,7 +155,7 @@ impl Console {
 
     pub(crate) fn process_control_rx(&mut self) -> bool {
         log::trace!("process_control_rx");
-        let DeviceState::Activated(ref mem) = self.device_state else {
+        let DeviceState::Activated(ref mem, _) = self.device_state else {
             unreachable!()
         };
         let mut raise_irq = false;
@@ -176,7 +172,7 @@ impl Console {
                         if let Err(e) =
                             self.queues[CONTROL_RXQ_INDEX].add_used(mem, head.index, n as u32)
                         {
-                            error!("failed to add used elements to the queue: {:?}", e);
+                            error!("failed to add used elements to the queue: {e:?}");
                         }
                     }
                     Err(e) => {
@@ -193,7 +189,7 @@ impl Console {
 
     pub(crate) fn process_control_tx(&mut self) -> bool {
         log::trace!("process_control_tx");
-        let DeviceState::Activated(ref mem) = self.device_state else {
+        let DeviceState::Activated(ref mem, ref interrupt) = self.device_state else {
             unreachable!()
         };
 
@@ -217,7 +213,7 @@ impl Console {
                 }
             };
             if let Err(e) = tx_queue.add_used(mem, head.index, size_of_val(&cmd) as u32) {
-                error!("failed to add used elements to the queue: {:?}", e);
+                error!("failed to add used elements to the queue: {e:?}");
             }
 
             log::trace!("VirtioConsoleControl cmd: {cmd:?}");
@@ -233,7 +229,7 @@ impl Console {
                 }
                 control_event::VIRTIO_CONSOLE_PORT_READY => {
                     if cmd.value != 1 {
-                        log::error!("Port initialization failed: {:?}", cmd);
+                        log::error!("Port initialization failed: {cmd:?}");
                         continue;
                     }
 
@@ -281,12 +277,12 @@ impl Console {
         }
 
         for port_id in ports_to_start {
-            log::trace!("Starting port io for port {}", port_id);
+            log::trace!("Starting port io for port {port_id}");
             self.ports[port_id].start(
                 mem.clone(),
                 self.queues[port_id_to_queue_idx(QueueDirection::Rx, port_id)].clone(),
                 self.queues[port_id_to_queue_idx(QueueDirection::Tx, port_id)].clone(),
-                self.irq.clone(),
+                interrupt.clone(),
                 self.control.clone(),
             );
         }
@@ -312,6 +308,10 @@ impl VirtioDevice for Console {
         uapi::VIRTIO_ID_CONSOLE
     }
 
+    fn device_name(&self) -> &str {
+        "console"
+    }
+
     fn queues(&self) -> &[VirtQueue] {
         &self.queues
     }
@@ -324,18 +324,6 @@ impl VirtioDevice for Console {
         &self.queue_events
     }
 
-    fn interrupt_evt(&self) -> &EventFd {
-        self.irq.interrupt_evt()
-    }
-
-    fn interrupt_status(&self) -> Arc<AtomicUsize> {
-        self.irq.interrupt_status()
-    }
-
-    fn set_irq_line(&mut self, irq: u32) {
-        self.irq.set_irq_line(irq)
-    }
-
     fn read_config(&self, offset: u64, mut data: &mut [u8]) {
         let config_slice = self.config.as_slice();
         let config_len = config_slice.len() as u64;
@@ -358,22 +346,18 @@ impl VirtioDevice for Console {
         );
     }
 
-    fn activate(&mut self, mem: GuestMemoryMmap) -> ActivateResult {
+    fn activate(&mut self, mem: GuestMemoryMmap, interrupt: InterruptTransport) -> ActivateResult {
         if self.activate_evt.write(1).is_err() {
             error!("Cannot write to activate_evt");
             return Err(ActivateError::BadActivate);
         }
-
-        self.device_state = DeviceState::Activated(mem);
+        self.device_state = DeviceState::Activated(mem, interrupt);
 
         Ok(())
     }
 
     fn is_activated(&self) -> bool {
-        match self.device_state {
-            DeviceState::Inactive => false,
-            DeviceState::Activated(_) => true,
-        }
+        self.device_state.is_activated()
     }
 
     fn reset(&mut self) -> bool {
