@@ -682,7 +682,7 @@ impl PassthroughFs {
     }
 
     fn do_open(&self, inode: Inode, mut flags: u32) -> io::Result<(Option<Handle>, OpenOptions)> {
-        debug!("do_open: {:?}", inode);
+        debug!("do_open: {inode:?}");
         if !self.cap_fowner {
             // O_NOATIME can only be used with CAP_FOWNER or if we are the file
             // owner. Not worth checking the latter, just drop it if we don't
@@ -939,7 +939,7 @@ impl FileSystem for PassthroughFs {
     }
 
     fn lookup(&self, _ctx: Context, parent: Inode, name: &CStr) -> io::Result<Entry> {
-        debug!("do_lookup: {:?}", name);
+        debug!("do_lookup: {name:?}");
         let init_name = unsafe { CStr::from_bytes_with_nul_unchecked(INIT_CSTR) };
 
         if self.init_inode != 0 && name == init_name {
@@ -1173,7 +1173,7 @@ impl FileSystem for PassthroughFs {
         _lock_owner: Option<u64>,
         _flags: u32,
     ) -> io::Result<usize> {
-        debug!("read: {:?}", inode);
+        debug!("read: {inode:?}");
         if inode == self.init_inode {
             let off: usize = offset
                 .try_into()
@@ -2017,7 +2017,7 @@ impl FileSystem for PassthroughFs {
 
         let addr = host_shm_base + moffset;
 
-        debug!("setupmapping: ino {:?} addr={:x} len={}", inode, addr, len);
+        debug!("setupmapping: ino {inode:?} addr={addr:x} len={len}");
 
         if inode == self.init_inode {
             let ret = unsafe {
@@ -2126,6 +2126,10 @@ impl FileSystem for PassthroughFs {
         const VIRTIO_IOC_EXIT_CODE_REQ: u32 =
             request_code_none!(VIRTIO_IOC_MAGIC, VIRTIO_IOC_TYPE_EXIT_CODE) as u32;
 
+        const VIRTIO_IOC_REMOVE_ROOT_DIR_CODE: u8 = 3;
+        const VIRTIO_IOC_REMOVE_ROOT_DIR_REQ: u32 =
+            request_code_none!(VIRTIO_IOC_MAGIC, VIRTIO_IOC_REMOVE_ROOT_DIR_CODE) as u32;
+
         match cmd {
             VIRTIO_IOC_EXPORT_FD_REQ => {
                 if out_size as usize != VIRTIO_IOC_EXPORT_FD_SIZE {
@@ -2160,6 +2164,10 @@ impl FileSystem for PassthroughFs {
                 exit_code.store(arg as i32, Ordering::SeqCst);
                 Ok(Vec::new())
             }
+            VIRTIO_IOC_REMOVE_ROOT_DIR_REQ => {
+                std::fs::remove_dir_all(&self.cfg.root_dir)?;
+                Ok(Vec::new())
+            }
             _ => Err(io::Error::from_raw_os_error(libc::EOPNOTSUPP)),
         }
     }
