@@ -1,6 +1,4 @@
 use std::io::Write;
-use std::result;
-use std::sync::atomic::{AtomicUsize, Ordering};
 use std::sync::{Arc, Mutex};
 
 use crossbeam_channel::{unbounded, Sender};
@@ -9,14 +7,15 @@ use vm_memory::{ByteValued, GuestMemoryMmap};
 
 use super::super::{
     fs::ExportTable, ActivateError, ActivateResult, DeviceState, GpuError, Queue as VirtQueue,
-    VirtioDevice, VirtioShmRegion, VIRTIO_MMIO_INT_VRING,
+    VirtioDevice, VirtioShmRegion,
 };
 use super::defs;
 use super::defs::uapi;
 use super::defs::uapi::virtio_gpu_config;
 use super::worker::Worker;
-use crate::legacy::IrqChip;
-use crate::Error as DeviceError;
+use crate::virtio::display::DisplayInfo;
+use crate::virtio::InterruptTransport;
+use krun_display::DisplayBackend;
 #[cfg(target_os = "macos")]
 use utils::worker_message::WorkerMessage;
 
@@ -28,6 +27,7 @@ pub(crate) const CUR_INDEX: usize = 1;
 // Supported features.
 pub(crate) const AVAIL_FEATURES: u64 = (1u64 << uapi::VIRTIO_F_VERSION_1)
     | (1u64 << uapi::VIRTIO_GPU_F_VIRGL)
+    | (1u64 << uapi::VIRTIO_GPU_F_EDID)
     | (1u64 << uapi::VIRTIO_GPU_F_RESOURCE_UUID)
     | (1u64 << uapi::VIRTIO_GPU_F_RESOURCE_BLOB)
     | (1u64 << uapi::VIRTIO_GPU_F_CONTEXT_INIT);
@@ -39,24 +39,24 @@ pub struct Gpu {
     pub(crate) queue_events: Vec<EventFd>,
     pub(crate) avail_features: u64,
     pub(crate) acked_features: u64,
-    pub(crate) interrupt_status: Arc<AtomicUsize>,
-    pub(crate) interrupt_evt: EventFd,
     pub(crate) activate_evt: EventFd,
     pub(crate) device_state: DeviceState,
     shm_region: Option<VirtioShmRegion>,
-    intc: Option<IrqChip>,
-    irq_line: Option<u32>,
     pub(crate) sender: Option<Sender<u64>>,
     virgl_flags: u32,
     #[cfg(target_os = "macos")]
     map_sender: Sender<WorkerMessage>,
     export_table: Option<ExportTable>,
+    displays: Box<[DisplayInfo]>,
+    display_backend: DisplayBackend<'static>,
 }
 
 impl Gpu {
     pub(crate) fn with_queues(
         queues: Vec<VirtQueue>,
         virgl_flags: u32,
+        displays: Box<[DisplayInfo]>,
+        display_backend: DisplayBackend<'static>,
         #[cfg(target_os = "macos")] map_sender: Sender<WorkerMessage>,
     ) -> super::Result<Gpu> {
         let mut queue_events = Vec::new();
@@ -75,23 +75,23 @@ impl Gpu {
             queue_events,
             avail_features: AVAIL_FEATURES,
             acked_features: 0,
-            interrupt_status: Arc::new(AtomicUsize::new(0)),
-            interrupt_evt: EventFd::new(utils::eventfd::EFD_NONBLOCK).map_err(GpuError::EventFd)?,
             activate_evt: EventFd::new(utils::eventfd::EFD_NONBLOCK).map_err(GpuError::EventFd)?,
             device_state: DeviceState::Inactive,
             shm_region: None,
-            intc: None,
-            irq_line: None,
             sender: None,
             virgl_flags,
             #[cfg(target_os = "macos")]
             map_sender,
             export_table: None,
+            displays,
+            display_backend,
         })
     }
 
     pub fn new(
         virgl_flags: u32,
+        displays: Box<[DisplayInfo]>,
+        display_backend: DisplayBackend<'static>,
         #[cfg(target_os = "macos")] map_sender: Sender<WorkerMessage>,
     ) -> super::Result<Gpu> {
         let queues: Vec<VirtQueue> = defs::QUEUE_SIZES
@@ -101,6 +101,8 @@ impl Gpu {
         Self::with_queues(
             queues,
             virgl_flags,
+            displays,
+            display_backend,
             #[cfg(target_os = "macos")]
             map_sender,
         )
@@ -110,10 +112,6 @@ impl Gpu {
         defs::GPU_DEV_ID
     }
 
-    pub fn set_intc(&mut self, intc: IrqChip) {
-        self.intc = Some(intc);
-    }
-
     pub fn set_shm_region(&mut self, shm_region: VirtioShmRegion) {
         debug!("virtio_gpu: set_shm_region");
         self.shm_region = Some(shm_region);
@@ -123,18 +121,6 @@ impl Gpu {
         self.export_table = Some(export_table);
     }
 
-    pub fn signal_used_queue(&self) -> result::Result<(), DeviceError> {
-        debug!("gpu: raising IRQ");
-        self.interrupt_status
-            .fetch_or(VIRTIO_MMIO_INT_VRING as usize, Ordering::SeqCst);
-        if let Some(intc) = &self.intc {
-            intc.lock()
-                .unwrap()
-                .set_irq(self.irq_line, Some(&self.interrupt_evt))?;
-        }
-        Ok(())
-    }
-
     /*
     pub fn process_ctl(&mut self) -> bool {
         debug!("gpu: process_ctl()");
@@ -208,6 +194,10 @@ impl VirtioDevice for Gpu {
         uapi::VIRTIO_ID_GPU
     }
 
+    fn device_name(&self) -> &str {
+        "gpu"
+    }
+
     fn queues(&self) -> &[VirtQueue] {
         &self.queues
     }
@@ -220,24 +210,11 @@ impl VirtioDevice for Gpu {
         &self.queue_events
     }
 
-    fn interrupt_evt(&self) -> &EventFd {
-        &self.interrupt_evt
-    }
-
-    fn interrupt_status(&self) -> Arc<AtomicUsize> {
-        self.interrupt_status.clone()
-    }
-
-    fn set_irq_line(&mut self, irq: u32) {
-        debug!("SET_IRQ_LINE (GPU)={}", irq);
-        self.irq_line = Some(irq);
-    }
-
     fn read_config(&self, offset: u64, mut data: &mut [u8]) {
         let config = virtio_gpu_config {
             events_read: 0,
             events_clear: 0,
-            num_scanouts: 0,
+            num_scanouts: self.displays.len() as u32,
             num_capsets: 5,
         };
 
@@ -262,7 +239,7 @@ impl VirtioDevice for Gpu {
         );
     }
 
-    fn activate(&mut self, mem: GuestMemoryMmap) -> ActivateResult {
+    fn activate(&mut self, mem: GuestMemoryMmap, interrupt: InterruptTransport) -> ActivateResult {
         if self.queues.len() != defs::NUM_QUEUES {
             error!(
                 "Cannot perform activate. Expected {} queue(s), got {}",
@@ -285,15 +262,14 @@ impl VirtioDevice for Gpu {
             receiver,
             mem.clone(),
             self.queue_ctl.clone(),
-            self.interrupt_status.clone(),
-            self.interrupt_evt.try_clone().unwrap(),
-            self.intc.clone(),
-            self.irq_line,
+            interrupt.clone(),
             shm_region,
             self.virgl_flags,
             #[cfg(target_os = "macos")]
             self.map_sender.clone(),
             self.export_table.take(),
+            self.displays.clone(),
+            self.display_backend,
         );
         worker.run();
 
@@ -304,16 +280,13 @@ impl VirtioDevice for Gpu {
             return Err(ActivateError::BadActivate);
         }
 
-        self.device_state = DeviceState::Activated(mem);
+        self.device_state = DeviceState::Activated(mem, interrupt);
 
         Ok(())
     }
 
     fn is_activated(&self) -> bool {
-        match self.device_state {
-            DeviceState::Inactive => false,
-            DeviceState::Activated(_) => true,
-        }
+        self.device_state.is_activated()
     }
 
     fn shm_region(&self) -> Option<&VirtioShmRegion> {
