@@ -1,13 +1,22 @@
 use std::collections::BTreeMap;
 use std::env;
+use std::io::IoSliceMut;
 #[cfg(target_os = "linux")]
 use std::os::fd::AsRawFd;
 use std::path::PathBuf;
-use std::sync::atomic::{AtomicUsize, Ordering};
 use std::sync::{Arc, Mutex};
 
+use super::super::Queue as VirtQueue;
+use super::protocol::GpuResponse::*;
+use super::protocol::{
+    GpuResponse, GpuResponsePlaneInfo, VirtioGpuResult, VIRTIO_GPU_BLOB_FLAG_CREATE_GUEST_HANDLE,
+    VIRTIO_GPU_BLOB_MEM_HOST3D, VIRTIO_GPU_MAX_SCANOUTS,
+};
 #[cfg(target_os = "macos")]
 use crossbeam_channel::{unbounded, Sender};
+use krun_display::{
+    DisplayBackend, DisplayBackendBasicFramebuffer, DisplayBackendInstance, Rect, ResourceFormat,
+};
 use libc::c_void;
 #[cfg(target_os = "macos")]
 use rutabaga_gfx::RUTABAGA_MEM_HANDLE_TYPE_APPLE;
@@ -25,23 +34,15 @@ use rutabaga_gfx::{
     RUTABAGA_CHANNEL_TYPE_PW, RUTABAGA_CHANNEL_TYPE_X11, RUTABAGA_MAP_ACCESS_MASK,
     RUTABAGA_MAP_ACCESS_READ, RUTABAGA_MAP_ACCESS_RW, RUTABAGA_MAP_ACCESS_WRITE,
 };
-use utils::eventfd::EventFd;
 #[cfg(target_os = "macos")]
 use utils::worker_message::WorkerMessage;
 use vm_memory::{GuestAddress, GuestMemory, GuestMemoryMmap, VolatileSlice};
 
-use super::super::Queue as VirtQueue;
-use super::protocol::GpuResponse::*;
-use super::protocol::{
-    GpuResponse, GpuResponsePlaneInfo, VirtioGpuResult, VIRTIO_GPU_BLOB_FLAG_CREATE_GUEST_HANDLE,
-    VIRTIO_GPU_BLOB_MEM_HOST3D,
-};
-
 use super::{GpuError, Result};
-use crate::legacy::IrqChip;
+use crate::virtio::display::DisplayInfo;
 use crate::virtio::fs::ExportTable;
 use crate::virtio::gpu::protocol::VIRTIO_GPU_FLAG_INFO_RING_IDX;
-use crate::virtio::{VirtioShmRegion, VIRTIO_MMIO_INT_VRING};
+use crate::virtio::{InterruptTransport, VirtioShmRegion};
 
 fn sglist_to_rutabaga_iovecs(
     vecs: &[(GuestAddress, usize)],
@@ -84,8 +85,35 @@ pub struct FenceState {
     completed_fences: BTreeMap<VirtioGpuRing, u64>,
 }
 
+#[derive(Copy, Clone, Debug, Default)]
+struct AssociatedScanouts(u32);
+
+impl AssociatedScanouts {
+    fn enable(&mut self, scanout_id: u32) {
+        self.0 |= 1 << scanout_id;
+    }
+
+    fn disable(&mut self, scanout_id: u32) {
+        self.0 ^= 1 << scanout_id;
+    }
+
+    const fn has_any_enabled(self) -> bool {
+        self.0 != 0
+    }
+
+    fn iter_enabled(self) -> impl Iterator<Item = u32> {
+        (0..VIRTIO_GPU_MAX_SCANOUTS).filter(move |i| ((self.0 >> i) & 1) == 1)
+    }
+}
+
+#[derive(Copy, Clone)]
 struct VirtioGpuResource {
-    size: u64,
+    id: u32,
+    width: u32,
+    height: u32,
+    scanouts: AssociatedScanouts,
+    format: Option<ResourceFormat>,
+    size: u64, // only for blob resources
     shmem_offset: Option<u64>,
     rutabaga_external_mapping: bool,
 }
@@ -93,21 +121,39 @@ struct VirtioGpuResource {
 impl VirtioGpuResource {
     /// Creates a new VirtioGpuResource with the given metadata.  Width and height are used by the
     /// display, while size is useful for hypervisor mapping.
-    pub fn new(_resource_id: u32, _width: u32, _height: u32, size: u64) -> VirtioGpuResource {
+    pub fn new(
+        resource_id: u32,
+        width: u32,
+        height: u32,
+        format: Option<ResourceFormat>,
+        size: u64,
+    ) -> VirtioGpuResource {
         VirtioGpuResource {
+            id: resource_id,
+            width,
+            height,
+            scanouts: Default::default(),
             size,
+            format,
             shmem_offset: None,
             rutabaga_external_mapping: false,
         }
     }
 }
 
+pub struct VirtioGpuScanout {
+    resource_id: u32,
+}
+
 pub struct VirtioGpu {
     rutabaga: Rutabaga,
     resources: BTreeMap<u32, VirtioGpuResource>,
     fence_state: Arc<Mutex<FenceState>>,
     #[cfg(target_os = "macos")]
     map_sender: Sender<WorkerMessage>,
+    scanouts: [Option<VirtioGpuScanout>; VIRTIO_GPU_MAX_SCANOUTS as usize],
+    displays: Box<[DisplayInfo]>,
+    display_backend: DisplayBackendInstance,
 }
 
 impl VirtioGpu {
@@ -115,10 +161,7 @@ impl VirtioGpu {
         mem: GuestMemoryMmap,
         queue_ctl: Arc<Mutex<VirtQueue>>,
         fence_state: Arc<Mutex<FenceState>>,
-        interrupt_status: Arc<AtomicUsize>,
-        interrupt_evt: EventFd,
-        intc: Option<IrqChip>,
-        irq_line: Option<u32>,
+        interrupt: InterruptTransport,
     ) -> RutabagaFenceHandler {
         RutabagaFenceHandler::new(move |completed_fence: RutabagaFence| {
             debug!(
@@ -152,16 +195,10 @@ impl VirtioGpu {
                     if let Err(e) =
                         queue.add_used(&mem, completed_desc.desc_index, completed_desc.len)
                     {
-                        error!("failed to add used elements to the queue: {:?}", e);
+                        error!("failed to add used elements to the queue: {e:?}");
                     }
 
-                    interrupt_status.fetch_or(VIRTIO_MMIO_INT_VRING as usize, Ordering::SeqCst);
-                    if let Some(intc) = &intc {
-                        if let Err(e) = intc.lock().unwrap().set_irq(irq_line, Some(&interrupt_evt))
-                        {
-                            error!("Failed to signal used queue: {:?}", e);
-                        }
-                    }
+                    interrupt.signal_used_queue();
                 } else {
                     i += 1;
                 }
@@ -177,13 +214,12 @@ impl VirtioGpu {
     pub fn new(
         mem: GuestMemoryMmap,
         queue_ctl: Arc<Mutex<VirtQueue>>,
-        interrupt_status: Arc<AtomicUsize>,
-        interrupt_evt: EventFd,
-        intc: Option<IrqChip>,
-        irq_line: Option<u32>,
+        interrupt: InterruptTransport,
         virgl_flags: u32,
         #[cfg(target_os = "macos")] map_sender: Sender<WorkerMessage>,
         export_table: Option<ExportTable>,
+        displays: Box<[DisplayInfo]>,
+        display_backend: DisplayBackend,
     ) -> Self {
         let xdg_runtime_dir = match env::var("XDG_RUNTIME_DIR") {
             Ok(dir) => dir,
@@ -193,7 +229,7 @@ impl VirtioGpu {
             Ok(display) => display,
             Err(_) => "wayland-0".to_string(),
         };
-        let path = PathBuf::from(format!("{}/{}", xdg_runtime_dir, wayland_display));
+        let path = PathBuf::from(format!("{xdg_runtime_dir}/{wayland_display}"));
 
         #[allow(unused_mut)]
         let mut rutabaga_channels: Vec<RutabagaChannel> = vec![RutabagaChannel {
@@ -204,7 +240,7 @@ impl VirtioGpu {
         #[cfg(target_os = "linux")]
         if let Ok(x_display) = env::var("DISPLAY") {
             if let Some(x_display) = x_display.strip_prefix(":") {
-                let x_path = PathBuf::from(format!("/tmp/.X11-unix/X{}", x_display));
+                let x_path = PathBuf::from(format!("/tmp/.X11-unix/X{x_display}"));
                 rutabaga_channels.push(RutabagaChannel {
                     base_channel: x_path,
                     channel_type: RUTABAGA_CHANNEL_TYPE_X11,
@@ -232,7 +268,6 @@ impl VirtioGpu {
             0,
         )
         .set_rutabaga_channels(rutabaga_channels_opt);
-
         let builder = if let Some(export_table) = export_table {
             builder.set_export_table(export_table)
         } else {
@@ -240,23 +275,23 @@ impl VirtioGpu {
         };
 
         let fence_state = Arc::new(Mutex::new(Default::default()));
-        let fence = Self::create_fence_handler(
-            mem,
-            queue_ctl.clone(),
-            fence_state.clone(),
-            interrupt_status,
-            interrupt_evt,
-            intc,
-            irq_line,
-        );
+        let fence =
+            Self::create_fence_handler(mem, queue_ctl.clone(), fence_state.clone(), interrupt);
         let rutabaga = builder
             .build(fence, None)
             .expect("Rutabaga initialization failed!");
 
+        let display_backend = display_backend
+            .create_instance()
+            .expect("Failed to create display backend instance!");
+
         Self {
             rutabaga,
             resources: Default::default(),
             fence_state,
+            scanouts: Default::default(),
+            displays,
+            display_backend,
             #[cfg(target_os = "macos")]
             map_sender,
         }
@@ -296,10 +331,19 @@ impl VirtioGpu {
         self.rutabaga
             .resource_create_3d(resource_id, resource_create_3d)?;
 
+        let format = ResourceFormat::try_from(resource_create_3d.format).ok();
+        if format.is_none() {
+            debug!(
+                "Unknown format {} for resource {}",
+                resource_create_3d.format, resource_id
+            );
+        }
+
         let resource = VirtioGpuResource::new(
             resource_id,
             resource_create_3d.width,
             resource_create_3d.height,
+            format,
             0,
         );
 
@@ -315,6 +359,14 @@ impl VirtioGpu {
             .remove(&resource_id)
             .ok_or(ErrInvalidResourceId)?;
 
+        if resource.scanouts.has_any_enabled() {
+            warn!(
+                "The driver requested unref_resource, but resource {resource_id} has \
+                     associated scanouts, refusing to delete the resource."
+            );
+            return Err(ErrUnspec);
+        }
+
         if resource.rutabaga_external_mapping {
             self.rutabaga.unmap(resource_id)?;
         }
@@ -323,12 +375,114 @@ impl VirtioGpu {
         Ok(OkNoData)
     }
 
+    pub fn set_scanout(
+        &mut self,
+        scanout_id: u32,
+        resource_id: u32,
+        width: u32,
+        height: u32,
+    ) -> VirtioGpuResult {
+        let scanout = self
+            .scanouts
+            .get_mut(scanout_id as usize)
+            .ok_or(ErrInvalidScanoutId)?;
+
+        // If a resource is already associated with this scanout, make sure to disable
+        // this scanout for that resource
+        if let Some(resource_id) = scanout.as_ref().map(|scanout| scanout.resource_id) {
+            let resource = self
+                .resources
+                .get_mut(&resource_id)
+                .ok_or(ErrInvalidResourceId)?;
+
+            resource.scanouts.disable(scanout_id);
+        }
+
+        // Virtio spec: "The driver can use resource_id = 0 to disable a scanout."
+        if resource_id == 0 {
+            debug!("Disabling scanout {scanout_id:?}");
+            *scanout = None;
+            self.display_backend.disable_scanout(scanout_id)?;
+            return Ok(OkNoData);
+        }
+
+        // Enable the scanout
+        let resource = self
+            .resources
+            .get_mut(&resource_id)
+            .ok_or(ErrInvalidResourceId)?;
+        resource.scanouts.enable(scanout_id);
+
+        let Some(format) = resource.format else {
+            warn!("Cannot use resource {resource_id} with unknown format for scanout");
+            return Err(ErrUnspec);
+        };
+
+        let display_info = self
+            .displays
+            .get(scanout_id as usize)
+            .ok_or(ErrInvalidScanoutId)?;
+
+        self.display_backend.configure_scanout(
+            scanout_id,
+            display_info.width,
+            display_info.height,
+            width,
+            height,
+            format,
+        )?;
+
+        *scanout = Some(VirtioGpuScanout { resource_id });
+        Ok(OkNoData)
+    }
+
+    fn read_2d_resource(
+        rutabaga: &mut Rutabaga,
+        resource: VirtioGpuResource,
+        output: &mut [u8],
+    ) -> VirtioGpuResult {
+        let transfer = Transfer3D {
+            x: 0,
+            y: 0,
+            z: 0,
+            w: resource.width,
+            h: resource.height,
+            d: 1,
+            level: 0,
+            stride: resource.width * ResourceFormat::BYTES_PER_PIXEL as u32,
+            layer_stride: 0,
+            offset: 0,
+        };
+
+        rutabaga
+            .transfer_read(0, resource.id, transfer, Some(IoSliceMut::new(output)))
+            .map_err(|e| format!("{e}"))
+            .unwrap();
+
+        Ok(OkNoData)
+    }
+
     /// If the resource is the scanout resource, flush it to the display.
-    pub fn flush_resource(&mut self, resource_id: u32) -> VirtioGpuResult {
+    pub fn flush_resource(&mut self, resource_id: u32, rect: Rect) -> VirtioGpuResult {
         if resource_id == 0 {
             return Ok(OkNoData);
         }
 
+        let resource = *self
+            .resources
+            .get(&resource_id)
+            .ok_or(ErrInvalidResourceId)?;
+
+        for scanout_id in resource.scanouts.iter_enabled() {
+            let (frame_id, buffer) = self.display_backend.alloc_frame(scanout_id)?;
+            if let Err(e) = Self::read_2d_resource(&mut self.rutabaga, resource, buffer) {
+                log::error!("Failed to read resource {resource_id} for scanout {scanout_id}: {e}");
+                return Err(ErrUnspec);
+            }
+            self.display_backend
+                .present_frame(scanout_id, frame_id, Some(&rect))?
+        }
+
         #[cfg(windows)]
         match self.rutabaga.resource_flush(resource_id) {
             Ok(_) => return Ok(OkNoData),
@@ -339,6 +493,25 @@ impl VirtioGpu {
         Ok(OkNoData)
     }
 
+    pub fn display_info(&self) -> VirtioGpuResult {
+        let display_info = self
+            .displays
+            .iter()
+            .map(|d| (d.width, d.height, true))
+            .collect();
+
+        Ok(OkDisplayInfo(display_info))
+    }
+
+    pub fn get_edid(&self, scanout_id: u32) -> VirtioGpuResult {
+        let display = self
+            .displays
+            .get(scanout_id as usize)
+            .ok_or(ErrInvalidScanoutId)?;
+
+        Ok(OkEdid(display.edid_bytes()))
+    }
+
     /// Copies data to host resource from the attached iovecs. Can also be used to flush caches.
     pub fn transfer_write(
         &mut self,
@@ -516,7 +689,7 @@ impl VirtioGpu {
             None,
         )?;
 
-        let resource = VirtioGpuResource::new(resource_id, 0, 0, resource_create_blob.size);
+        let resource = VirtioGpuResource::new(resource_id, 0, 0, None, resource_create_blob.size);
 
         // Rely on rutabaga to check for duplicate resource ids.
         self.resources.insert(resource_id, resource);
@@ -771,3 +944,44 @@ impl VirtioGpu {
         Ok(OkNoData)
     }
 }
+#[cfg(test)]
+mod test {
+    use crate::virtio::gpu::protocol::VIRTIO_GPU_MAX_SCANOUTS;
+
+    #[test]
+    fn test_virtio_gpu_associated_scanouts() {
+        use super::AssociatedScanouts;
+
+        let mut scanouts = AssociatedScanouts::default();
+
+        assert!(!scanouts.has_any_enabled());
+        assert_eq!(scanouts.iter_enabled().next(), None);
+
+        scanouts.enable(1);
+        assert!(scanouts.has_any_enabled());
+        scanouts.disable(1);
+        assert!(!scanouts.has_any_enabled());
+
+        (0..VIRTIO_GPU_MAX_SCANOUTS).for_each(|scanout| scanouts.enable(scanout));
+        assert!(scanouts.has_any_enabled());
+        assert_eq!(
+            scanouts.iter_enabled().collect::<Vec<u32>>(),
+            (0..VIRTIO_GPU_MAX_SCANOUTS).collect::<Vec<u32>>()
+        );
+
+        (0..VIRTIO_GPU_MAX_SCANOUTS)
+            .filter(|&i| i % 2 == 0)
+            .for_each(|scanout| scanouts.disable(scanout));
+        assert_eq!(
+            scanouts.iter_enabled().collect::<Vec<u32>>(),
+            (1..VIRTIO_GPU_MAX_SCANOUTS)
+                .step_by(2)
+                .collect::<Vec<u32>>()
+        );
+
+        (0..VIRTIO_GPU_MAX_SCANOUTS)
+            .filter(|&i| i % 2 != 0)
+            .for_each(|scanout| scanouts.disable(scanout));
+        assert!(!scanouts.has_any_enabled());
+    }
+}
