@@ -1,7 +1,6 @@
 use std::io::Read;
-use std::sync::atomic::{AtomicUsize, Ordering};
 use std::sync::{Arc, Mutex};
-use std::{result, thread};
+use std::thread;
 
 use crossbeam_channel::Receiver;
 #[cfg(target_os = "macos")]
@@ -10,37 +9,36 @@ use rutabaga_gfx::{
     ResourceCreate3D, ResourceCreateBlob, RutabagaFence, Transfer3D,
     RUTABAGA_PIPE_BIND_RENDER_TARGET, RUTABAGA_PIPE_TEXTURE_2D,
 };
-use utils::eventfd::EventFd;
 #[cfg(target_os = "macos")]
 use utils::worker_message::WorkerMessage;
 use vm_memory::{GuestAddress, GuestMemoryMmap};
 
 use super::super::descriptor_utils::{Reader, Writer};
-use super::super::{GpuError, Queue as VirtQueue, VIRTIO_MMIO_INT_VRING};
+use super::super::{GpuError, Queue as VirtQueue};
 use super::protocol::{
     virtio_gpu_ctrl_hdr, virtio_gpu_mem_entry, GpuCommand, GpuResponse, VirtioGpuResult,
 };
 use super::virtio_gpu::VirtioGpu;
-use crate::legacy::IrqChip;
+use crate::virtio::display::DisplayInfo;
 use crate::virtio::fs::ExportTable;
 use crate::virtio::gpu::protocol::{VIRTIO_GPU_FLAG_FENCE, VIRTIO_GPU_FLAG_INFO_RING_IDX};
 use crate::virtio::gpu::virtio_gpu::VirtioGpuRing;
-use crate::virtio::VirtioShmRegion;
-use crate::Error as DeviceError;
+use crate::virtio::{InterruptTransport, VirtioShmRegion};
+use krun_display::DisplayBackend;
+use krun_display::Rect;
 
 pub struct Worker {
     receiver: Receiver<u64>,
     mem: GuestMemoryMmap,
     queue_ctl: Arc<Mutex<VirtQueue>>,
-    interrupt_status: Arc<AtomicUsize>,
-    interrupt_evt: EventFd,
-    intc: Option<IrqChip>,
-    irq_line: Option<u32>,
+    interrupt: InterruptTransport,
     shm_region: VirtioShmRegion,
     virgl_flags: u32,
     #[cfg(target_os = "macos")]
     map_sender: Sender<WorkerMessage>,
     export_table: Option<ExportTable>,
+    displays: Box<[DisplayInfo]>,
+    display_backend: DisplayBackend<'static>,
 }
 
 impl Worker {
@@ -49,28 +47,26 @@ impl Worker {
         receiver: Receiver<u64>,
         mem: GuestMemoryMmap,
         queue_ctl: Arc<Mutex<VirtQueue>>,
-        interrupt_status: Arc<AtomicUsize>,
-        interrupt_evt: EventFd,
-        intc: Option<IrqChip>,
-        irq_line: Option<u32>,
+        interrupt: InterruptTransport,
         shm_region: VirtioShmRegion,
         virgl_flags: u32,
         #[cfg(target_os = "macos")] map_sender: Sender<WorkerMessage>,
         export_table: Option<ExportTable>,
+        displays: Box<[DisplayInfo]>,
+        display_backend: DisplayBackend<'static>,
     ) -> Self {
         Self {
             receiver,
             mem,
             queue_ctl,
-            interrupt_status,
-            interrupt_evt,
-            intc,
-            irq_line,
+            interrupt,
             shm_region,
             virgl_flags,
             #[cfg(target_os = "macos")]
             map_sender,
             export_table,
+            displays,
+            display_backend,
         }
     }
 
@@ -85,38 +81,25 @@ impl Worker {
         let mut virtio_gpu = VirtioGpu::new(
             self.mem.clone(),
             self.queue_ctl.clone(),
-            self.interrupt_status.clone(),
-            self.interrupt_evt.try_clone().unwrap(),
-            self.intc.clone(),
-            self.irq_line,
+            self.interrupt.clone(),
             self.virgl_flags,
             #[cfg(target_os = "macos")]
             self.map_sender.clone(),
             self.export_table.take(),
+            self.displays.clone(),
+            self.display_backend,
         );
 
         loop {
             let _ = self.receiver.recv().unwrap();
             if self.process_queue(&mut virtio_gpu, 0) {
-                if let Err(e) = self.signal_used_queue() {
-                    error!("Error signaling queue: {:?}", e);
+                if let Err(e) = self.interrupt.try_signal_used_queue() {
+                    error!("Error signaling queue: {e:?}");
                 }
             }
         }
     }
 
-    pub fn signal_used_queue(&self) -> result::Result<(), DeviceError> {
-        debug!("gpu: raising IRQ");
-        self.interrupt_status
-            .fetch_or(VIRTIO_MMIO_INT_VRING as usize, Ordering::SeqCst);
-        if let Some(intc) = &self.intc {
-            intc.lock()
-                .unwrap()
-                .set_irq(self.irq_line, Some(&self.interrupt_evt))?;
-        }
-        Ok(())
-    }
-
     fn process_gpu_command(
         &mut self,
         virtio_gpu: &mut VirtioGpu,
@@ -128,9 +111,8 @@ impl Worker {
         virtio_gpu.force_ctx_0();
 
         match cmd {
-            GpuCommand::GetDisplayInfo(_) => {
-                panic!("virtio_gpu: GpuCommand::GetDisplayInfo unimplemented");
-            }
+            GpuCommand::GetDisplayInfo => virtio_gpu.display_info(),
+            GpuCommand::GetEdid(info) => virtio_gpu.get_edid(info.scanout),
             GpuCommand::ResourceCreate2d(info) => {
                 let resource_id = info.resource_id;
 
@@ -150,10 +132,21 @@ impl Worker {
                 virtio_gpu.resource_create_3d(resource_id, resource_create_3d)
             }
             GpuCommand::ResourceUnref(info) => virtio_gpu.unref_resource(info.resource_id),
-            GpuCommand::SetScanout(_info) => {
-                panic!("virtio_gpu: GpuCommand::SetScanout unimplemented");
+            GpuCommand::SetScanout(info) => virtio_gpu.set_scanout(
+                info.scanout_id,
+                info.resource_id,
+                info.r.width,
+                info.r.height,
+            ),
+            GpuCommand::ResourceFlush(info) => {
+                let rect = Rect {
+                    x: info.r.x,
+                    y: info.r.y,
+                    width: info.r.width,
+                    height: info.r.height,
+                };
+                virtio_gpu.flush_resource(info.resource_id, rect)
             }
-            GpuCommand::ResourceFlush(info) => virtio_gpu.flush_resource(info.resource_id),
             GpuCommand::TransferToHost2d(info) => {
                 let resource_id = info.resource_id;
                 let transfer = Transfer3D::new_2d(info.r.x, info.r.y, info.r.width, info.r.height);
@@ -176,7 +169,7 @@ impl Worker {
                     }
                     virtio_gpu.attach_backing(info.resource_id, mem, vecs)
                 } else {
-                    error!("missing data for command {:?}", cmd);
+                    error!("missing data for command {cmd:?}");
                     Err(GpuResponse::ErrUnspec)
                 }
             }
@@ -366,13 +359,13 @@ impl Worker {
                         ctrl_hdr = Some(hdr);
                         gpu_cmd = Some(cmd);
                     }
-                    Err(e) => debug!("descriptor decode error: {:?}", e),
+                    Err(e) => debug!("descriptor decode error: {e:?}"),
                 }
 
                 let mut gpu_response = match resp {
                     Ok(gpu_response) => gpu_response,
                     Err(gpu_response) => {
-                        debug!("{:?} -> {:?}", gpu_cmd, gpu_response);
+                        debug!("{gpu_cmd:?} -> {gpu_response:?}");
                         gpu_response
                     }
                 };
@@ -401,7 +394,7 @@ impl Worker {
                             gpu_response = match virtio_gpu.create_fence(fence) {
                                 Ok(_) => gpu_response,
                                 Err(fence_resp) => {
-                                    warn!("create_fence {} -> {:?}", fence_id, fence_resp);
+                                    warn!("create_fence {fence_id} -> {fence_resp:?}");
                                     fence_resp
                                 }
                             };
@@ -412,7 +405,7 @@ impl Worker {
                     // fence is complete.
                     match gpu_response.encode(flags, fence_id, ctx_id, ring_idx, &mut writer) {
                         Ok(l) => len = l,
-                        Err(e) => debug!("ctrl queue response encode error: {:?}", e),
+                        Err(e) => debug!("ctrl queue response encode error: {e:?}"),
                     }
 
                     if flags & VIRTIO_GPU_FLAG_FENCE != 0 {
@@ -432,7 +425,7 @@ impl Worker {
                         .unwrap()
                         .add_used(&mem, head.index, len)
                     {
-                        error!("failed to add used elements to the queue: {:?}", e);
+                        error!("failed to add used elements to the queue: {e:?}");
                     }
                     used_any = true;
                 }
