@@ -4,11 +4,12 @@
 // Portions Copyright 2017 The Chromium OS Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the THIRD-PARTY file.
-use crate::legacy::IrqChip;
 use crate::virtio::net::{Error, Result};
 use crate::virtio::net::{QUEUE_SIZES, RX_INDEX, TX_INDEX};
 use crate::virtio::queue::Error as QueueError;
-use crate::virtio::{ActivateResult, DeviceState, Queue, VirtioDevice, TYPE_NET};
+use crate::virtio::{
+    ActivateError, ActivateResult, DeviceState, InterruptTransport, Queue, VirtioDevice, TYPE_NET,
+};
 use crate::Error as DeviceError;
 
 use super::backend::{ReadError, WriteError};
@@ -18,13 +19,8 @@ use std::cmp;
 use std::io::Write;
 use std::os::fd::RawFd;
 use std::path::PathBuf;
-use std::sync::atomic::AtomicUsize;
-use std::sync::Arc;
 use utils::eventfd::{EventFd, EFD_NONBLOCK};
-use virtio_bindings::virtio_net::{
-    VIRTIO_NET_F_CSUM, VIRTIO_NET_F_GUEST_CSUM, VIRTIO_NET_F_GUEST_TSO4, VIRTIO_NET_F_GUEST_UFO,
-    VIRTIO_NET_F_HOST_TSO4, VIRTIO_NET_F_HOST_UFO, VIRTIO_NET_F_MAC,
-};
+use virtio_bindings::virtio_net::VIRTIO_NET_F_MAC;
 use virtio_bindings::virtio_ring::VIRTIO_RING_F_EVENT_IDX;
 use vm_memory::{ByteValued, GuestMemoryError, GuestMemoryMmap};
 
@@ -65,8 +61,12 @@ unsafe impl ByteValued for VirtioNetConfig {}
 
 #[derive(Clone)]
 pub enum VirtioNetBackend {
-    Passt(RawFd),
-    Gvproxy(PathBuf),
+    UnixstreamFd(RawFd),
+    UnixstreamPath(PathBuf),
+    UnixgramFd(RawFd),
+    UnixgramPath(PathBuf, bool),
+    #[cfg(target_os = "linux")]
+    Tap(String),
 }
 
 pub struct Net {
@@ -79,26 +79,20 @@ pub struct Net {
     queues: Vec<Queue>,
     queue_evts: Vec<EventFd>,
 
-    interrupt_status: Arc<AtomicUsize>,
-    interrupt_evt: EventFd,
-
     pub(crate) device_state: DeviceState,
 
-    intc: Option<IrqChip>,
-    irq_line: Option<u32>,
-
     config: VirtioNetConfig,
 }
 
 impl Net {
     /// Create a new virtio network device using the backend
-    pub fn new(id: String, cfg_backend: VirtioNetBackend, mac: [u8; 6]) -> Result<Self> {
-        let avail_features = (1 << VIRTIO_NET_F_GUEST_CSUM)
-            | (1 << VIRTIO_NET_F_CSUM)
-            | (1 << VIRTIO_NET_F_GUEST_TSO4)
-            | (1 << VIRTIO_NET_F_HOST_TSO4)
-            | (1 << VIRTIO_NET_F_GUEST_UFO)
-            | (1 << VIRTIO_NET_F_HOST_UFO)
+    pub fn new(
+        id: String,
+        cfg_backend: VirtioNetBackend,
+        mac: [u8; 6],
+        features: u32,
+    ) -> Result<Self> {
+        let avail_features = features as u64
             | (1 << VIRTIO_NET_F_MAC)
             | (1 << VIRTIO_RING_F_EVENT_IDX)
             | (1 << VIRTIO_F_VERSION_1);
@@ -125,15 +119,7 @@ impl Net {
 
             queues,
             queue_evts,
-
-            interrupt_status: Arc::new(AtomicUsize::new(0)),
-            interrupt_evt: EventFd::new(EFD_NONBLOCK).map_err(Error::EventFd)?,
-
             device_state: DeviceState::Inactive,
-
-            intc: None,
-            irq_line: None,
-
             config,
         })
     }
@@ -142,10 +128,6 @@ impl Net {
     pub fn id(&self) -> &str {
         &self.id
     }
-
-    pub fn set_intc(&mut self, intc: IrqChip) {
-        self.intc = Some(intc);
-    }
 }
 
 impl VirtioDevice for Net {
@@ -165,6 +147,10 @@ impl VirtioDevice for Net {
         TYPE_NET
     }
 
+    fn device_name(&self) -> &str {
+        "net"
+    }
+
     fn queues(&self) -> &[Queue] {
         &self.queues
     }
@@ -177,19 +163,6 @@ impl VirtioDevice for Net {
         &self.queue_evts
     }
 
-    fn interrupt_evt(&self) -> &EventFd {
-        &self.interrupt_evt
-    }
-
-    fn interrupt_status(&self) -> Arc<AtomicUsize> {
-        self.interrupt_status.clone()
-    }
-
-    fn set_irq_line(&mut self, irq: u32) {
-        debug!("SET_IRQ_LINE (NET)={}", irq);
-        self.irq_line = Some(irq);
-    }
-
     fn read_config(&self, offset: u64, mut data: &mut [u8]) {
         let config_slice = self.config.as_slice();
         let config_len = config_slice.len() as u64;
@@ -212,7 +185,7 @@ impl VirtioDevice for Net {
         );
     }
 
-    fn activate(&mut self, mem: GuestMemoryMmap) -> ActivateResult {
+    fn activate(&mut self, mem: GuestMemoryMmap, interrupt: InterruptTransport) -> ActivateResult {
         let event_idx: bool = (self.acked_features & (1 << VIRTIO_RING_F_EVENT_IDX)) != 0;
         self.queues[RX_INDEX].set_event_idx(event_idx);
         self.queues[TX_INDEX].set_event_idx(event_idx);
@@ -222,26 +195,31 @@ impl VirtioDevice for Net {
             .iter()
             .map(|e| e.try_clone().unwrap())
             .collect();
-        let worker = NetWorker::new(
+
+        match NetWorker::new(
             self.queues.clone(),
             queue_evts,
-            self.interrupt_status.clone(),
-            self.interrupt_evt.try_clone().unwrap(),
-            self.intc.clone(),
-            self.irq_line,
+            interrupt.clone(),
             mem.clone(),
+            self.acked_features,
             self.cfg_backend.clone(),
-        );
-        worker.run();
-
-        self.device_state = DeviceState::Activated(mem);
-        Ok(())
+        ) {
+            Ok(worker) => {
+                worker.run();
+                self.device_state = DeviceState::Activated(mem, interrupt);
+                Ok(())
+            }
+            Err(err) => {
+                error!(
+                    "Error activating virtio-net ({}) backend: {err:?}",
+                    self.id()
+                );
+                Err(ActivateError::BadActivate)
+            }
+        }
     }
 
     fn is_activated(&self) -> bool {
-        match self.device_state {
-            DeviceState::Inactive => false,
-            DeviceState::Activated(_) => true,
-        }
+        self.device_state.is_activated()
     }
 }
