@@ -1,43 +1,26 @@
-use crate::legacy::IrqChip;
-use crate::virtio::net::gvproxy::Gvproxy;
-use crate::virtio::net::passt::Passt;
+use crate::virtio::net::backend::ConnectError;
+#[cfg(target_os = "linux")]
+use crate::virtio::net::tap::Tap;
+use crate::virtio::net::unixgram::Unixgram;
+use crate::virtio::net::unixstream::Unixstream;
 use crate::virtio::net::{MAX_BUFFER_SIZE, QUEUE_SIZE, RX_INDEX, TX_INDEX};
-use crate::virtio::{Queue, VIRTIO_MMIO_INT_VRING};
-use crate::Error as DeviceError;
+use crate::virtio::{InterruptTransport, Queue};
 
 use super::backend::{NetBackend, ReadError, WriteError};
 use super::device::{FrontendError, RxError, TxError, VirtioNetBackend};
+use super::vnet_hdr_len;
 
 use std::os::fd::AsRawFd;
-use std::sync::atomic::AtomicUsize;
-use std::sync::atomic::Ordering;
-use std::sync::Arc;
 use std::thread;
-use std::{cmp, mem, result};
+use std::{cmp, result};
 use utils::epoll::{ControlOperation, Epoll, EpollEvent, EventSet};
 use utils::eventfd::EventFd;
-use virtio_bindings::virtio_net::virtio_net_hdr_v1;
 use vm_memory::{Bytes, GuestAddress, GuestMemoryMmap};
 
-fn vnet_hdr_len() -> usize {
-    mem::size_of::<virtio_net_hdr_v1>()
-}
-
-// This initializes to all 0 the virtio_net_hdr part of a buf and return the length of the header
-// https://docs.oasis-open.org/virtio/virtio/v1.1/csprd01/virtio-v1.1-csprd01.html#x1-2050006
-fn write_virtio_net_hdr(buf: &mut [u8]) -> usize {
-    let len = vnet_hdr_len();
-    buf[0..len].fill(0);
-    len
-}
-
 pub struct NetWorker {
     queues: Vec<Queue>,
     queue_evts: Vec<EventFd>,
-    interrupt_status: Arc<AtomicUsize>,
-    interrupt_evt: EventFd,
-    intc: Option<IrqChip>,
-    irq_line: Option<u32>,
+    interrupt: InterruptTransport,
 
     mem: GuestMemoryMmap,
     backend: Box<dyn NetBackend + Send>,
@@ -56,30 +39,37 @@ impl NetWorker {
     pub fn new(
         queues: Vec<Queue>,
         queue_evts: Vec<EventFd>,
-        interrupt_status: Arc<AtomicUsize>,
-        interrupt_evt: EventFd,
-        intc: Option<IrqChip>,
-        irq_line: Option<u32>,
+        interrupt: InterruptTransport,
         mem: GuestMemoryMmap,
+        _vnet_features: u64,
         cfg_backend: VirtioNetBackend,
-    ) -> Self {
+    ) -> Result<Self, ConnectError> {
         let backend = match cfg_backend {
-            VirtioNetBackend::Passt(fd) => Box::new(Passt::new(fd)) as Box<dyn NetBackend + Send>,
-            VirtioNetBackend::Gvproxy(path) => {
-                Box::new(Gvproxy::new(path).unwrap()) as Box<dyn NetBackend + Send>
+            VirtioNetBackend::UnixstreamFd(fd) => {
+                Box::new(Unixstream::new(fd)) as Box<dyn NetBackend + Send>
+            }
+            VirtioNetBackend::UnixstreamPath(path) => {
+                Box::new(Unixstream::open(path)?) as Box<dyn NetBackend + Send>
+            }
+            VirtioNetBackend::UnixgramFd(fd) => {
+                Box::new(Unixgram::new(fd)) as Box<dyn NetBackend + Send>
+            }
+            VirtioNetBackend::UnixgramPath(path, vfkit_magic) => {
+                Box::new(Unixgram::open(path, vfkit_magic)?) as Box<dyn NetBackend + Send>
+            }
+            #[cfg(target_os = "linux")]
+            VirtioNetBackend::Tap(tap_name) => {
+                Box::new(Tap::new(tap_name, _vnet_features)?) as Box<dyn NetBackend + Send>
             }
         };
 
-        Self {
+        Ok(Self {
             queues,
             queue_evts,
-            interrupt_status,
-            interrupt_evt,
-            intc,
-            irq_line,
 
             mem,
             backend,
+            interrupt,
 
             rx_frame_buf: [0u8; MAX_BUFFER_SIZE],
             rx_frame_buf_len: 0,
@@ -88,7 +78,7 @@ impl NetWorker {
             tx_frame_buf: [0u8; MAX_BUFFER_SIZE],
             tx_frame_len: 0,
             tx_iovec: Vec::with_capacity(QUEUE_SIZE as usize),
-        }
+        })
     }
 
     pub fn run(self) {
@@ -156,16 +146,14 @@ impl NetWorker {
                             }
                             _ => {
                                 log::warn!(
-                                    "Received unknown event: {:?} from fd: {:?}",
-                                    event_set,
-                                    source
+                                    "Received unknown event: {event_set:?} from fd: {source:?}"
                                 );
                             }
                         }
                     }
                 }
                 Err(e) => {
-                    debug!("vsock: failed to consume muxer epoll event: {}", e);
+                    debug!("vsock: failed to consume muxer epoll event: {e}");
                 }
             }
         }
@@ -173,16 +161,16 @@ impl NetWorker {
 
     pub(crate) fn process_rx_queue_event(&mut self) {
         if let Err(e) = self.queue_evts[RX_INDEX].read() {
-            log::error!("Failed to get rx event from queue: {:?}", e);
+            log::error!("Failed to get rx event from queue: {e:?}");
         }
         if let Err(e) = self.queues[RX_INDEX].disable_notification(&self.mem) {
-            error!("error disabling queue notifications: {:?}", e);
+            error!("error disabling queue notifications: {e:?}");
         }
         if let Err(e) = self.process_rx() {
             log::error!("Failed to process rx: {e:?} (triggered by queue event)")
         };
         if let Err(e) = self.queues[RX_INDEX].enable_notification(&self.mem) {
-            error!("error disabling queue notifications: {:?}", e);
+            error!("error disabling queue notifications: {e:?}");
         }
     }
 
@@ -197,13 +185,13 @@ impl NetWorker {
 
     pub(crate) fn process_backend_socket_readable(&mut self) {
         if let Err(e) = self.queues[RX_INDEX].enable_notification(&self.mem) {
-            error!("error disabling queue notifications: {:?}", e);
+            error!("error disabling queue notifications: {e:?}");
         }
         if let Err(e) = self.process_rx() {
             log::error!("Failed to process rx: {e:?} (triggered by backend socket readable)");
         };
         if let Err(e) = self.queues[RX_INDEX].disable_notification(&self.mem) {
-            error!("error disabling queue notifications: {:?}", e);
+            error!("error disabling queue notifications: {e:?}");
         }
     }
 
@@ -255,7 +243,9 @@ impl NetWorker {
         // At this point we processed as many Rx frames as possible.
         // We have to wake the guest if at least one descriptor chain has been used.
         if signal_queue {
-            self.signal_used_queue().map_err(RxError::DeviceError)?;
+            self.interrupt
+                .try_signal_used_queue()
+                .map_err(RxError::DeviceError)?;
         }
 
         result
@@ -324,7 +314,7 @@ impl NetWorker {
                         read_count += limit - read_count;
                     }
                     Err(e) => {
-                        log::error!("Failed to read slice: {:?}", e);
+                        log::error!("Failed to read slice: {e:?}");
                         read_count = 0;
                         break;
                     }
@@ -373,23 +363,14 @@ impl NetWorker {
         }
 
         if raise_irq && tx_queue.needs_notification(&self.mem).unwrap() {
-            self.signal_used_queue().map_err(TxError::DeviceError)?;
+            self.interrupt
+                .try_signal_used_queue()
+                .map_err(TxError::DeviceError)?;
         }
 
         Ok(())
     }
 
-    fn signal_used_queue(&mut self) -> result::Result<(), DeviceError> {
-        self.interrupt_status
-            .fetch_or(VIRTIO_MMIO_INT_VRING as usize, Ordering::SeqCst);
-        if let Some(intc) = &self.intc {
-            intc.lock()
-                .unwrap()
-                .set_irq(self.irq_line, Some(&self.interrupt_evt))?;
-        }
-        Ok(())
-    }
-
     // Copies a single frame from `self.rx_frame_buf` into the guest.
     fn write_frame_to_guest_impl(&mut self) -> result::Result<(), FrontendError> {
         let mut result: std::result::Result<(), FrontendError> = Ok(());
@@ -418,7 +399,7 @@ impl NetWorker {
                     frame_slice = &frame_slice[len..];
                 }
                 Err(e) => {
-                    log::error!("Failed to write slice: {:?}", e);
+                    log::error!("Failed to write slice: {e:?}");
                     result = Err(FrontendError::GuestMemory(e));
                     break;
                 }
@@ -462,10 +443,7 @@ impl NetWorker {
 
     /// Fills self.rx_frame_buf with an ethernet frame from backend and prepends virtio_net_hdr to it
     fn read_into_rx_frame_buf_from_backend(&mut self) -> result::Result<(), ReadError> {
-        let mut len = 0;
-        len += write_virtio_net_hdr(&mut self.rx_frame_buf);
-        len += self.backend.read_frame(&mut self.rx_frame_buf[len..])?;
-        self.rx_frame_buf_len = len;
+        self.rx_frame_buf_len = self.backend.read_frame(&mut self.rx_frame_buf)?;
         Ok(())
     }
 }
