@@ -1,18 +1,12 @@
-use std::result;
-use std::sync::atomic::{AtomicUsize, Ordering};
-use std::sync::Arc;
-
 use rand::{rngs::OsRng, RngCore};
 use utils::eventfd::EventFd;
 use vm_memory::{Bytes, GuestMemoryMmap};
 
 use super::super::{
     ActivateError, ActivateResult, DeviceState, Queue as VirtQueue, RngError, VirtioDevice,
-    VIRTIO_MMIO_INT_VRING,
 };
 use super::{defs, defs::uapi};
-use crate::legacy::IrqChip;
-use crate::Error as DeviceError;
+use crate::virtio::InterruptTransport;
 
 // Request queue.
 pub(crate) const REQ_INDEX: usize = 0;
@@ -20,21 +14,13 @@ pub(crate) const REQ_INDEX: usize = 0;
 // Supported features.
 pub(crate) const AVAIL_FEATURES: u64 = 1 << uapi::VIRTIO_F_VERSION_1 as u64;
 
-#[derive(Copy, Clone, Debug, Default)]
-#[repr(C, packed)]
-pub struct VirtioRng {}
-
 pub struct Rng {
     pub(crate) queues: Vec<VirtQueue>,
     pub(crate) queue_events: Vec<EventFd>,
     pub(crate) avail_features: u64,
     pub(crate) acked_features: u64,
-    pub(crate) interrupt_status: Arc<AtomicUsize>,
-    pub(crate) interrupt_evt: EventFd,
     pub(crate) activate_evt: EventFd,
     pub(crate) device_state: DeviceState,
-    intc: Option<IrqChip>,
-    irq_line: Option<u32>,
 }
 
 impl Rng {
@@ -50,12 +36,8 @@ impl Rng {
             queue_events,
             avail_features: AVAIL_FEATURES,
             acked_features: 0,
-            interrupt_status: Arc::new(AtomicUsize::new(0)),
-            interrupt_evt: EventFd::new(utils::eventfd::EFD_NONBLOCK).map_err(RngError::EventFd)?,
             activate_evt: EventFd::new(utils::eventfd::EFD_NONBLOCK).map_err(RngError::EventFd)?,
             device_state: DeviceState::Inactive,
-            intc: None,
-            irq_line: None,
         })
     }
 
@@ -71,26 +53,10 @@ impl Rng {
         defs::RNG_DEV_ID
     }
 
-    pub fn set_intc(&mut self, intc: IrqChip) {
-        self.intc = Some(intc);
-    }
-
-    pub fn signal_used_queue(&self) -> result::Result<(), DeviceError> {
-        debug!("rng: raising IRQ");
-        self.interrupt_status
-            .fetch_or(VIRTIO_MMIO_INT_VRING as usize, Ordering::SeqCst);
-        if let Some(intc) = &self.intc {
-            intc.lock()
-                .unwrap()
-                .set_irq(self.irq_line, Some(&self.interrupt_evt))?;
-        }
-        Ok(())
-    }
-
     pub fn process_req(&mut self) -> bool {
         debug!("rng: process_req()");
         let mem = match self.device_state {
-            DeviceState::Activated(ref mem) => mem,
+            DeviceState::Activated(ref mem, _) => mem,
             // This should never happen, it's been already validated in the event handler.
             DeviceState::Inactive => unreachable!(),
         };
@@ -104,7 +70,7 @@ impl Rng {
                 let mut rand_bytes = vec![0u8; desc.len as usize];
                 OsRng.fill_bytes(&mut rand_bytes);
                 if let Err(e) = mem.write_slice(&rand_bytes[..], desc.addr) {
-                    error!("Failed to write slice: {:?}", e);
+                    error!("Failed to write slice: {e:?}");
                     self.queues[REQ_INDEX].go_to_previous_position();
                     break;
                 }
@@ -113,7 +79,7 @@ impl Rng {
 
             have_used = true;
             if let Err(e) = self.queues[REQ_INDEX].add_used(mem, index, written) {
-                error!("failed to add used elements to the queue: {:?}", e);
+                error!("failed to add used elements to the queue: {e:?}");
             }
         }
 
@@ -138,6 +104,10 @@ impl VirtioDevice for Rng {
         uapi::VIRTIO_ID_RNG
     }
 
+    fn device_name(&self) -> &str {
+        "rng"
+    }
+
     fn queues(&self) -> &[VirtQueue] {
         &self.queues
     }
@@ -150,19 +120,6 @@ impl VirtioDevice for Rng {
         &self.queue_events
     }
 
-    fn interrupt_evt(&self) -> &EventFd {
-        &self.interrupt_evt
-    }
-
-    fn interrupt_status(&self) -> Arc<AtomicUsize> {
-        self.interrupt_status.clone()
-    }
-
-    fn set_irq_line(&mut self, irq: u32) {
-        debug!("SET_IRQ_LINE (RNG)={}", irq);
-        self.irq_line = Some(irq);
-    }
-
     fn read_config(&self, _offset: u64, _data: &mut [u8]) {
         error!("rng: invalid request to read config space");
     }
@@ -175,7 +132,7 @@ impl VirtioDevice for Rng {
         );
     }
 
-    fn activate(&mut self, mem: GuestMemoryMmap) -> ActivateResult {
+    fn activate(&mut self, mem: GuestMemoryMmap, interrupt: InterruptTransport) -> ActivateResult {
         if self.queues.len() != defs::NUM_QUEUES {
             error!(
                 "Cannot perform activate. Expected {} queue(s), got {}",
@@ -190,16 +147,13 @@ impl VirtioDevice for Rng {
             return Err(ActivateError::BadActivate);
         }
 
-        self.device_state = DeviceState::Activated(mem);
+        self.device_state = DeviceState::Activated(mem, interrupt);
 
         Ok(())
     }
 
     fn is_activated(&self) -> bool {
-        match self.device_state {
-            DeviceState::Inactive => false,
-            DeviceState::Activated(_) => true,
-        }
+        self.device_state.is_activated()
     }
 
     fn reset(&mut self) -> bool {
