@@ -60,12 +60,12 @@ fn proxy_fd_create(id: u64) -> Result<RawFd, ProxyError> {
         Ok(flags) => match OFlag::from_bits(flags) {
             Some(flags) => {
                 if let Err(e) = fcntl(fd, FcntlArg::F_SETFL(flags | OFlag::O_NONBLOCK)) {
-                    warn!("error switching to non-blocking: id={}, err={}", id, e);
+                    warn!("error switching to non-blocking: id={id}, err={e}");
                 }
             }
-            None => error!("invalid fd flags id={}", id),
+            None => error!("invalid fd flags id={id}"),
         },
-        Err(e) => error!("couldn't obtain fd flags id={}, err={}", id, e),
+        Err(e) => error!("couldn't obtain fd flags id={id}, err={e}"),
     };
 
     #[cfg(target_os = "macos")]
@@ -132,10 +132,7 @@ impl UnixProxy {
         queue: Arc<Mutex<VirtQueue>>,
         rxq: Arc<Mutex<MuxerRxQ>>,
     ) -> Self {
-        debug!(
-            "new_reverse: id={} local_port={} peer_port={}",
-            id, local_port, peer_port
-        );
+        debug!("new_reverse: id={id} local_port={local_port} peer_port={peer_port}");
         UnixProxy {
             id,
             cid,
@@ -223,7 +220,7 @@ impl UnixProxy {
 
             match recv(self.fd, &mut buf[..max_len], MsgFlags::MSG_DONTWAIT) {
                 Ok(cnt) => {
-                    debug!("vsock: unix: recv cnt={}", cnt);
+                    debug!("vsock: unix: recv cnt={cnt}");
                     if cnt > 0 {
                         debug!("vsock: tcp: recv rx_cnt={}", self.rx_cnt);
                         RecvPkt::Read(cnt)
@@ -232,7 +229,7 @@ impl UnixProxy {
                     }
                 }
                 Err(e) => {
-                    debug!("vsock: tcp: recv_pkt: recv error: {:?}", e);
+                    debug!("vsock: tcp: recv_pkt: recv error: {e:?}");
                     RecvPkt::Error
                 }
             }
@@ -267,7 +264,7 @@ impl UnixProxy {
                     RecvPkt::Error => 0,
                 },
                 Err(e) => {
-                    debug!("vsock: tcp: recv_pkt: RX queue error: {:?}", e);
+                    debug!("vsock: tcp: recv_pkt: RX queue error: {e:?}");
                     0
                 }
             };
@@ -283,12 +280,12 @@ impl UnixProxy {
                     len, self.push_cnt
                 );
                 if let Err(e) = queue.add_used(&self.mem, head.index, len as u32) {
-                    error!("failed to add used elements to the queue: {:?}", e);
+                    error!("failed to add used elements to the queue: {e:?}");
                 }
             }
         }
 
-        debug!("vsock: tcp: recv_pkt: have_used={}", have_used);
+        debug!("vsock: tcp: recv_pkt: have_used={have_used}");
         (have_used, wait_credit)
     }
 
@@ -335,7 +332,7 @@ impl Proxy for UnixProxy {
                 0
             }
             Err(e) => {
-                debug!("vsock: UnixProxy: Error connecting: {}", e);
+                debug!("vsock: UnixProxy: Error connecting: {e}");
                 #[cfg(target_os = "macos")]
                 let errno = -linux_errno_raw(nix::errno::errno());
                 #[cfg(target_os = "linux")]
@@ -435,7 +432,7 @@ impl Proxy for UnixProxy {
             update.signal_queue = true;
         }
 
-        debug!("vsock: tcp_proxy: sendmsg ret={}", ret);
+        debug!("vsock: tcp_proxy: sendmsg ret={ret}");
 
         update
     }
@@ -525,7 +522,7 @@ impl Proxy for UnixProxy {
         };
 
         if let Err(e) = shutdown(self.fd, how) {
-            warn!("error sending shutdown to socket: {}", e);
+            warn!("error sending shutdown to socket: {e}");
         }
     }
 
@@ -625,7 +622,7 @@ impl AsRawFd for UnixProxy {
 impl Drop for UnixProxy {
     fn drop(&mut self) {
         if let Err(e) = close(self.fd) {
-            warn!("error closing proxy fd: {}", e);
+            warn!("error closing proxy fd: {e}");
         }
     }
 }
@@ -726,7 +723,7 @@ impl AsRawFd for UnixAcceptorProxy {
 impl Drop for UnixAcceptorProxy {
     fn drop(&mut self) {
         if let Err(e) = close(self.fd) {
-            warn!("error closing proxy fd: {}", e);
+            warn!("error closing proxy fd: {e}");
         }
     }
 }
