@@ -126,7 +126,7 @@ impl Display for Error {
 
         match self {
             EnableEL2 => write!(f, "Error enabling EL2 mode in HVF"),
-            FindSymbol(ref err) => write!(f, "Couldn't find symbol in HVF library: {}", err),
+            FindSymbol(ref err) => write!(f, "Couldn't find symbol in HVF library: {err}"),
             MemoryMap => write!(f, "Error registering memory region in HVF"),
             MemoryUnmap => write!(f, "Error unregistering memory region in HVF"),
             NestedCheck => write!(
@@ -143,8 +143,7 @@ impl Display for Error {
             VcpuSetRegister => write!(f, "Error setting HVF vCPU register"),
             VcpuSetSystemRegister(reg, val) => write!(
                 f,
-                "Error setting HVF vCPU system register 0x{:#x} to 0x{:#x}",
-                reg, val
+                "Error setting HVF vCPU system register 0x{reg:#x} to 0x{val:#x}"
             ),
             VcpuSetVtimerMask => write!(f, "Error setting HVF vCPU vtimer mask"),
             VmCreate => write!(f, "Error creating HVF VM instance"),
@@ -222,7 +221,7 @@ pub fn check_nested_virt() -> Result<bool, Error> {
     let mut el2_supported: bool = false;
     let ret = unsafe { (get_el2_supported.unwrap())(&mut el2_supported) };
     if ret != HV_SUCCESS {
-        error!("hv_vm_config_get_el2_supported failed: {:?}", ret);
+        error!("hv_vm_config_get_el2_supported failed: {ret:?}");
         return Err(Error::NestedCheck);
     }
 
@@ -344,6 +343,8 @@ impl HvfVcpu<'_> {
         };
         #[cfg(target_arch = "x86_64")]
         let cntfrq = 0u64;
+        #[cfg(target_arch = "riscv64")]
+        let cntfrq = 0u64;
 
         let ret = unsafe {
             hv_vcpu_create(
@@ -522,7 +523,7 @@ impl HvfVcpu<'_> {
         }
     }
 
-    fn handle_psci_request(&self) -> Result<VcpuExit, Error> {
+    fn handle_psci_request(&self) -> Result<VcpuExit<'_>, Error> {
         match self.read_reg(hv_reg_t_HV_REG_X0)? {
             0x8400_0000 /* QEMU_PSCI_0_2_FN_PSCI_VERSION */ => {
                 self.write_reg(hv_reg_t_HV_REG_X0, 2)?;
@@ -545,11 +546,11 @@ impl HvfVcpu<'_> {
                 self.write_reg(hv_reg_t_HV_REG_X0, 0)?;
                 Ok(VcpuExit::CpuOn(mpidr, entry, context_id))
             }
-            val => panic!("Unexpected val={}", val)
+            val => panic!("Unexpected val={val}")
         }
     }
 
-    pub fn run(&mut self, vcpu_list: Arc<dyn Vcpus>) -> Result<VcpuExit, Error> {
+    pub fn run(&mut self, vcpu_list: Arc<dyn Vcpus>) -> Result<VcpuExit<'_>, Error> {
         let pending_irq = vcpu_list.has_pending_irq(self.vcpuid);
 
         if let Some(mmio_read) = self.pending_mmio_read.take() {
