@@ -1,13 +1,33 @@
 #[macro_use]
 extern crate log;
 
+use crossbeam_channel::unbounded;
+#[cfg(feature = "blk")]
+use devices::virtio::block::ImageType;
+#[cfg(feature = "gpu")]
+use devices::virtio::gpu::display::DisplayInfo;
+#[cfg(feature = "net")]
+use devices::virtio::net::device::VirtioNetBackend;
+#[cfg(feature = "blk")]
+use devices::virtio::CacheType;
+use env_logger::{Env, Target};
+#[cfg(feature = "gpu")]
+use krun_display::DisplayBackend;
+use libc::c_char;
+#[cfg(feature = "net")]
+use libc::c_int;
+use libc::size_t;
+use once_cell::sync::Lazy;
+use polly::event_manager::EventManager;
+#[cfg(all(feature = "blk", not(feature = "tee")))]
+use rand::distr::{Alphanumeric, SampleString};
 use std::collections::hash_map::Entry;
 use std::collections::HashMap;
 use std::convert::TryInto;
 use std::env;
-use std::ffi::CStr;
 #[cfg(target_os = "linux")]
 use std::ffi::CString;
+use std::ffi::{c_void, CStr};
 use std::fs::File;
 #[cfg(target_os = "linux")]
 use std::os::fd::AsRawFd;
@@ -18,24 +38,10 @@ use std::sync::atomic::{AtomicI32, Ordering};
 #[cfg(not(feature = "efi"))]
 use std::sync::LazyLock;
 use std::sync::Mutex;
-
-use crossbeam_channel::unbounded;
-#[cfg(feature = "blk")]
-use devices::virtio::block::ImageType;
-#[cfg(feature = "net")]
-use devices::virtio::net::device::VirtioNetBackend;
-#[cfg(feature = "blk")]
-use devices::virtio::CacheType;
-use env_logger::{Env, Target};
-#[cfg(not(feature = "efi"))]
-use libc::size_t;
-use libc::{c_char, c_int};
-use once_cell::sync::Lazy;
-use polly::event_manager::EventManager;
 use utils::eventfd::EventFd;
 use vmm::resources::VmResources;
 #[cfg(feature = "blk")]
-use vmm::vmm_config::block::BlockDeviceConfig;
+use vmm::vmm_config::block::{BlockDeviceConfig, BlockRootConfig};
 use vmm::vmm_config::boot_source::{BootSourceConfig, DEFAULT_KERNEL_CMDLINE};
 #[cfg(not(feature = "tee"))]
 use vmm::vmm_config::external_kernel::{ExternalKernel, KernelFormat};
@@ -50,16 +56,26 @@ use vmm::vmm_config::machine_config::VmConfig;
 use vmm::vmm_config::net::NetworkInterfaceConfig;
 use vmm::vmm_config::vsock::VsockDeviceConfig;
 
+#[cfg(feature = "nitro")]
+use nitro::enclaves::NitroEnclave;
+
+#[cfg(feature = "gpu")]
+use devices::virtio::display::{DisplayInfoEdid, PhysicalSize, MAX_DISPLAYS};
+#[cfg(feature = "nitro")]
+use nitro_enclaves::launch::StartFlags;
+
 // Value returned on success. We use libc's errors otherwise.
 const KRUN_SUCCESS: i32 = 0;
 // Maximum number of arguments/environment variables we allow
 const MAX_ARGS: usize = 4096;
 
 // krunfw library name for each context
-#[cfg(all(target_os = "linux", not(feature = "amd-sev")))]
+#[cfg(all(target_os = "linux", not(feature = "tee")))]
 const KRUNFW_NAME: &str = "libkrunfw.so.4";
 #[cfg(all(target_os = "linux", feature = "amd-sev"))]
 const KRUNFW_NAME: &str = "libkrunfw-sev.so.4";
+#[cfg(all(target_os = "linux", feature = "tdx"))]
+const KRUNFW_NAME: &str = "libkrunfw-tdx.so.4";
 #[cfg(all(target_os = "macos", not(feature = "efi")))]
 const KRUNFW_NAME: &str = "libkrunfw.4.dylib";
 
@@ -105,24 +121,13 @@ impl KrunfwBindings {
     }
 }
 
-#[derive(Default)]
-struct TsiConfig {
-    port_map: Option<HashMap<u16, u16>>,
-}
-
-enum NetworkConfig {
-    Tsi(TsiConfig),
+#[derive(Clone)]
+#[cfg(feature = "net")]
+enum LegacyNetworkConfig {
     VirtioNetPasst(RawFd),
     VirtioNetGvproxy(PathBuf),
 }
 
-impl Default for NetworkConfig {
-    /// Default network mode is TSI, for backwards compatibility
-    fn default() -> Self {
-        NetworkConfig::Tsi(Default::default())
-    }
-}
-
 #[derive(Default)]
 struct ContextConfig {
     #[cfg(not(feature = "efi"))]
@@ -133,14 +138,20 @@ struct ContextConfig {
     env: Option<String>,
     args: Option<String>,
     rlimits: Option<String>,
-    net_cfg: NetworkConfig,
-    mac: Option<[u8; 6]>,
+    #[cfg(feature = "net")]
+    legacy_net_cfg: Option<LegacyNetworkConfig>,
+    #[cfg(feature = "net")]
+    legacy_mac: Option<[u8; 6]>,
+    net_index: u8,
+    tsi_port_map: Option<HashMap<u16, u16>>,
     #[cfg(feature = "blk")]
     block_cfgs: Vec<BlockDeviceConfig>,
     #[cfg(feature = "blk")]
     root_block_cfg: Option<BlockDeviceConfig>,
     #[cfg(feature = "blk")]
     data_block_cfg: Option<BlockDeviceConfig>,
+    #[cfg(feature = "blk")]
+    block_root: Option<BlockRootConfig>,
     #[cfg(feature = "tee")]
     tee_config_file: Option<PathBuf>,
     unix_ipc_port_map: Option<HashMap<u32, (PathBuf, bool)>>,
@@ -151,6 +162,10 @@ struct ContextConfig {
     console_output: Option<PathBuf>,
     vmm_uid: Option<libc::uid_t>,
     vmm_gid: Option<libc::gid_t>,
+    #[cfg(feature = "nitro")]
+    nitro_image_path: Option<PathBuf>,
+    #[cfg(feature = "nitro")]
+    nitro_start_flags: StartFlags,
 }
 
 impl ContextConfig {
@@ -176,6 +191,34 @@ impl ContextConfig {
         }
     }
 
+    #[cfg(all(feature = "blk", not(feature = "tee")))]
+    fn set_block_root(&mut self, device: String, fstype: Option<String>, options: Option<String>) {
+        self.block_root = Some(BlockRootConfig {
+            device,
+            fstype,
+            options,
+        });
+    }
+
+    fn get_block_root(&self) -> String {
+        #[cfg(feature = "blk")]
+        match &self.block_root {
+            Some(block_root) => {
+                let mut res = format!("KRUN_BLOCK_ROOT_DEVICE={}", block_root.device);
+                if let Some(fstype) = &block_root.fstype {
+                    res += &format!(" KRUN_BLOCK_ROOT_FSTYPE={}", fstype);
+                }
+                if let Some(options) = &block_root.options {
+                    res += &format!(" KRUN_BLOCK_ROOT_OPTIONS={}", options);
+                }
+                res
+            }
+            None => "".to_string(),
+        }
+        #[cfg(not(feature = "blk"))]
+        "".to_string()
+    }
+
     fn set_env(&mut self, env: String) {
         self.env = Some(env);
     }
@@ -241,23 +284,18 @@ impl ContextConfig {
         }
     }
 
-    fn set_net_cfg(&mut self, net_cfg: NetworkConfig) {
-        self.net_cfg = net_cfg;
-    }
-
+    #[cfg(feature = "net")]
     fn set_net_mac(&mut self, mac: [u8; 6]) {
-        self.mac = Some(mac);
+        self.legacy_mac = Some(mac);
     }
 
     fn set_port_map(&mut self, new_port_map: HashMap<u16, u16>) -> Result<(), ()> {
-        match &mut self.net_cfg {
-            NetworkConfig::Tsi(tsi_config) => {
-                tsi_config.port_map.replace(new_port_map);
-                Ok(())
-            }
-            NetworkConfig::VirtioNetPasst(_) => Err(()),
-            NetworkConfig::VirtioNetGvproxy(_) => Err(()),
+        if self.net_index != 0 {
+            return Err(());
         }
+
+        self.tsi_port_map.replace(new_port_map);
+        Ok(())
     }
 
     #[cfg(feature = "tee")]
@@ -295,6 +333,61 @@ impl ContextConfig {
     fn set_vmm_gid(&mut self, vmm_gid: libc::gid_t) {
         self.vmm_gid = Some(vmm_gid);
     }
+
+    #[cfg(feature = "nitro")]
+    fn set_nitro_image(&mut self, image_path: PathBuf) {
+        self.nitro_image_path = Some(image_path);
+    }
+
+    #[cfg(feature = "nitro")]
+    fn set_nitro_start_flags(&mut self, start_flags: StartFlags) {
+        self.nitro_start_flags = start_flags;
+    }
+}
+
+#[cfg(feature = "nitro")]
+impl TryFrom<ContextConfig> for NitroEnclave {
+    type Error = i32;
+
+    fn try_from(ctx: ContextConfig) -> Result<Self, Self::Error> {
+        let vm_config = ctx.vmr.vm_config();
+
+        let Some(mem_size_mib) = vm_config.mem_size_mib else {
+            error!("memory size not configured");
+            return Err(-libc::EINVAL);
+        };
+
+        let Some(vcpus) = vm_config.vcpu_count else {
+            error!("vCPU count not configured");
+            return Err(-libc::EINVAL);
+        };
+
+        let Some(image_path) = ctx.nitro_image_path else {
+            error!("nitro image not configured");
+            return Err(-libc::EINVAL);
+        };
+
+        let Ok(image) = File::open(&image_path) else {
+            error!("unable to open {}", image_path.display());
+            return Err(-libc::EINVAL);
+        };
+
+        Ok(Self {
+            image,
+            mem_size_mib,
+            vcpus,
+            start_flags: ctx.nitro_start_flags,
+        })
+    }
+}
+
+// TODO: Use this everywhere instead of the manual match
+#[allow(dead_code)]
+fn with_cfg(ctx_id: u32, f: impl FnOnce(&mut ContextConfig) -> i32) -> i32 {
+    match CTX_MAP.lock().unwrap().entry(ctx_id) {
+        Entry::Occupied(mut ctx_cfg) => f(ctx_cfg.get_mut()),
+        Entry::Vacant(_) => -libc::ENOENT,
+    }
 }
 
 static CTX_MAP: Lazy<Mutex<HashMap<u32, ContextConfig>>> = Lazy::new(|| Mutex::new(HashMap::new()));
@@ -673,21 +766,103 @@ pub unsafe extern "C" fn krun_set_data_disk(ctx_id: u32, c_disk_path: *const c_c
     KRUN_SUCCESS
 }
 
+/*
+ * Send the VFKIT magic after establishing the connection,
+ * as required by gvproxy in vfkit mode.
+ */
+#[cfg(feature = "net")]
+const NET_FLAG_VFKIT: u32 = 1 << 0;
+
+/* Taken from uapi/linux/virtio_net.h */
+#[cfg(feature = "net")]
+const NET_FEATURE_CSUM: u32 = 1 << 0;
+#[cfg(feature = "net")]
+const NET_FEATURE_GUEST_CSUM: u32 = 1 << 1;
+#[cfg(feature = "net")]
+const NET_FEATURE_GUEST_TSO4: u32 = 1 << 7;
+#[cfg(feature = "net")]
+const NET_FEATURE_GUEST_TSO6: u32 = 1 << 8;
+#[cfg(feature = "net")]
+const NET_FEATURE_GUEST_UFO: u32 = 1 << 10;
+#[cfg(feature = "net")]
+const NET_FEATURE_HOST_TSO4: u32 = 1 << 11;
+#[cfg(feature = "net")]
+const NET_FEATURE_HOST_TSO6: u32 = 1 << 12;
+#[cfg(feature = "net")]
+const NET_FEATURE_HOST_UFO: u32 = 1 << 14;
+/*
+ * These are the flags enabled by default on each virtio-net instance
+ * before the introduction of "krun_add_net_*". They are now used in
+ * the legacy API ("krun_set_passt_fd" and "krun_set_gvproxy_path")
+ * for compatiblity reasons.
+ */
+#[cfg(feature = "net")]
+const NET_COMPAT_FEATURES: u32 = NET_FEATURE_CSUM
+    | NET_FEATURE_GUEST_CSUM
+    | NET_FEATURE_GUEST_TSO4
+    | NET_FEATURE_GUEST_UFO
+    | NET_FEATURE_HOST_TSO4
+    | NET_FEATURE_HOST_UFO;
+#[cfg(feature = "net")]
+const NET_ALL_FEATURES: u32 = NET_FEATURE_CSUM
+    | NET_FEATURE_GUEST_CSUM
+    | NET_FEATURE_GUEST_TSO4
+    | NET_FEATURE_GUEST_TSO6
+    | NET_FEATURE_GUEST_UFO
+    | NET_FEATURE_HOST_TSO4
+    | NET_FEATURE_HOST_TSO6
+    | NET_FEATURE_HOST_UFO;
+
 #[allow(clippy::missing_safety_doc)]
 #[no_mangle]
-pub unsafe extern "C" fn krun_set_passt_fd(ctx_id: u32, fd: c_int) -> i32 {
-    if fd < 0 {
+#[cfg(feature = "net")]
+pub unsafe extern "C" fn krun_add_net_unixstream(
+    ctx_id: u32,
+    c_path: *const c_char,
+    fd: c_int,
+    c_mac: *const u8,
+    features: u32,
+    flags: u32,
+) -> i32 {
+    let path = if !c_path.is_null() {
+        match CStr::from_ptr(c_path).to_str() {
+            Ok(path) => Some(PathBuf::from(path)),
+            Err(_) => None,
+        }
+    } else {
+        None
+    };
+
+    if fd >= 0 && path.is_some() {
+        return -libc::EINVAL;
+    }
+    if fd < 0 && path.is_none() {
+        return -libc::EINVAL;
+    }
+    let backend = if let Some(path) = path {
+        VirtioNetBackend::UnixstreamPath(path)
+    } else {
+        VirtioNetBackend::UnixstreamFd(fd)
+    };
+
+    let mac: [u8; 6] = match slice::from_raw_parts(c_mac, 6).try_into() {
+        Ok(m) => m,
+        Err(_) => return -libc::EINVAL,
+    };
+
+    /* The unixstream backend doesn't support any flags */
+    if flags != 0 {
         return -libc::EINVAL;
     }
 
-    if cfg!(not(feature = "net")) {
-        return -libc::ENOTSUP;
+    if (features & !NET_ALL_FEATURES) != 0 {
+        return -libc::EINVAL;
     }
 
     match CTX_MAP.lock().unwrap().entry(ctx_id) {
         Entry::Occupied(mut ctx_cfg) => {
             let cfg = ctx_cfg.get_mut();
-            cfg.set_net_cfg(NetworkConfig::VirtioNetPasst(fd));
+            create_virtio_net(cfg, backend, mac, features);
         }
         Entry::Vacant(_) => return -libc::ENOENT,
     }
@@ -696,11 +871,153 @@ pub unsafe extern "C" fn krun_set_passt_fd(ctx_id: u32, fd: c_int) -> i32 {
 
 #[allow(clippy::missing_safety_doc)]
 #[no_mangle]
+#[cfg(feature = "net")]
+pub unsafe extern "C" fn krun_add_net_unixgram(
+    ctx_id: u32,
+    c_path: *const c_char,
+    fd: c_int,
+    c_mac: *const u8,
+    features: u32,
+    flags: u32,
+) -> i32 {
+    let path = if !c_path.is_null() {
+        match CStr::from_ptr(c_path).to_str() {
+            Ok(path) => Some(PathBuf::from(path)),
+            Err(_) => None,
+        }
+    } else {
+        None
+    };
+
+    if fd >= 0 && path.is_some() {
+        return -libc::EINVAL;
+    }
+    if fd < 0 && path.is_none() {
+        return -libc::EINVAL;
+    }
+
+    let mac: [u8; 6] = match slice::from_raw_parts(c_mac, 6).try_into() {
+        Ok(m) => m,
+        Err(_) => return -libc::EINVAL,
+    };
+
+    if (features & !NET_ALL_FEATURES) != 0 {
+        return -libc::EINVAL;
+    }
+
+    if (flags & !NET_FLAG_VFKIT) != 0 {
+        return -libc::EINVAL;
+    }
+    let send_vfkit_magic: bool = flags & NET_FLAG_VFKIT != 0;
+
+    let backend = if let Some(path) = path {
+        VirtioNetBackend::UnixgramPath(path, send_vfkit_magic)
+    } else {
+        VirtioNetBackend::UnixgramFd(fd)
+    };
+
+    match CTX_MAP.lock().unwrap().entry(ctx_id) {
+        Entry::Occupied(mut ctx_cfg) => {
+            let cfg = ctx_cfg.get_mut();
+            create_virtio_net(cfg, backend, mac, features);
+        }
+        Entry::Vacant(_) => return -libc::ENOENT,
+    }
+    KRUN_SUCCESS
+}
+
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+#[cfg(all(target_os = "linux", feature = "net"))]
+pub unsafe extern "C" fn krun_add_net_tap(
+    ctx_id: u32,
+    c_tap_name: *const c_char,
+    c_mac: *const u8,
+    features: u32,
+    flags: u32,
+) -> i32 {
+    let tap_name = match CStr::from_ptr(c_tap_name).to_str() {
+        Ok(tap_name) => tap_name.to_string(),
+        Err(e) => {
+            debug!("Error parsing tap_name: {e:?}");
+            return -libc::EINVAL;
+        }
+    };
+
+    let mac: [u8; 6] = match slice::from_raw_parts(c_mac, 6).try_into() {
+        Ok(m) => m,
+        Err(_) => return -libc::EINVAL,
+    };
+
+    if (features & !NET_ALL_FEATURES) != 0 {
+        return -libc::EINVAL;
+    }
+
+    if features & (NET_FEATURE_GUEST_TSO4 | NET_FEATURE_GUEST_TSO6 | NET_FEATURE_GUEST_UFO) != 0
+        && features & NET_FEATURE_GUEST_CSUM == 0
+    {
+        debug!("Network tap backend requires GUEST_CSUM to be requested if any of GUEST_TSO4, GUEST_TSO6 and/or GUEST_UFO are required");
+        return -libc::EINVAL;
+    }
+
+    /* The tap backend doesn't support any flags */
+    if flags != 0 {
+        return -libc::EINVAL;
+    }
+
+    match CTX_MAP.lock().unwrap().entry(ctx_id) {
+        Entry::Occupied(mut ctx_cfg) => {
+            let cfg = ctx_cfg.get_mut();
+            create_virtio_net(cfg, VirtioNetBackend::Tap(tap_name), mac, features);
+        }
+        Entry::Vacant(_) => return -libc::ENOENT,
+    }
+    KRUN_SUCCESS
+}
+
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+#[cfg(all(not(target_os = "linux"), feature = "net"))]
+pub unsafe extern "C" fn krun_add_net_tap(
+    _ctx_id: u32,
+    _c_tap_name: *const c_char,
+    _c_mac: *const u8,
+    _features: u32,
+    _flags: u32,
+) -> i32 {
+    -libc::EINVAL
+}
+
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+#[cfg(feature = "net")]
+pub unsafe extern "C" fn krun_set_passt_fd(ctx_id: u32, fd: c_int) -> i32 {
+    if fd < 0 {
+        return -libc::EINVAL;
+    }
+
+    match CTX_MAP.lock().unwrap().entry(ctx_id) {
+        Entry::Occupied(mut ctx_cfg) => {
+            let cfg = ctx_cfg.get_mut();
+            // The legacy interface only supports a single network interface.
+            if cfg.net_index != 0 {
+                return -libc::EINVAL;
+            }
+            cfg.legacy_net_cfg = Some(LegacyNetworkConfig::VirtioNetPasst(fd));
+        }
+        Entry::Vacant(_) => return -libc::ENOENT,
+    }
+    KRUN_SUCCESS
+}
+
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+#[cfg(feature = "net")]
 pub unsafe extern "C" fn krun_set_gvproxy_path(ctx_id: u32, c_path: *const c_char) -> i32 {
     let path_str = match CStr::from_ptr(c_path).to_str() {
         Ok(path) => path,
         Err(e) => {
-            debug!("Error parsing gvproxy_path: {:?}", e);
+            debug!("Error parsing gvproxy_path: {e:?}");
             return -libc::EINVAL;
         }
     };
@@ -710,7 +1027,11 @@ pub unsafe extern "C" fn krun_set_gvproxy_path(ctx_id: u32, c_path: *const c_cha
     match CTX_MAP.lock().unwrap().entry(ctx_id) {
         Entry::Occupied(mut ctx_cfg) => {
             let cfg = ctx_cfg.get_mut();
-            cfg.set_net_cfg(NetworkConfig::VirtioNetGvproxy(path));
+            // The legacy interface only supports a single network interface.
+            if cfg.net_index != 0 {
+                return -libc::EINVAL;
+            }
+            cfg.legacy_net_cfg = Some(LegacyNetworkConfig::VirtioNetGvproxy(path));
         }
         Entry::Vacant(_) => return -libc::ENOENT,
     }
@@ -719,11 +1040,8 @@ pub unsafe extern "C" fn krun_set_gvproxy_path(ctx_id: u32, c_path: *const c_cha
 
 #[allow(clippy::missing_safety_doc)]
 #[no_mangle]
+#[cfg(feature = "net")]
 pub unsafe extern "C" fn krun_set_net_mac(ctx_id: u32, c_mac: *const u8) -> i32 {
-    if cfg!(not(feature = "net")) {
-        return -libc::ENOTSUP;
-    }
-
     let mac: [u8; 6] = match slice::from_raw_parts(c_mac, 6).try_into() {
         Ok(m) => m,
         Err(_) => return -libc::EINVAL,
@@ -781,7 +1099,7 @@ pub unsafe extern "C" fn krun_set_port_map(ctx_id: u32, c_port_map: *const *cons
         Entry::Occupied(mut ctx_cfg) => {
             let cfg = ctx_cfg.get_mut();
             if cfg.set_port_map(port_map).is_err() {
-                return -libc::ENOTSUP;
+                return -libc::EINVAL;
             }
         }
         Entry::Vacant(_) => return -libc::ENOENT,
@@ -869,7 +1187,7 @@ pub unsafe extern "C" fn krun_set_exec(
     let exec_path = match CStr::from_ptr(c_exec_path).to_str() {
         Ok(path) => path,
         Err(e) => {
-            debug!("Error parsing exec_path: {:?}", e);
+            debug!("Error parsing exec_path: {e:?}");
             return -libc::EINVAL;
         }
     };
@@ -879,7 +1197,7 @@ pub unsafe extern "C" fn krun_set_exec(
         match collapse_str_array(argv_array) {
             Ok(s) => s,
             Err(e) => {
-                debug!("Error parsing args: {:?}", e);
+                debug!("Error parsing args: {e:?}");
                 return -libc::EINVAL;
             }
         }
@@ -892,7 +1210,7 @@ pub unsafe extern "C" fn krun_set_exec(
         match collapse_str_array(envp_array) {
             Ok(s) => s,
             Err(e) => {
-                debug!("Error parsing args: {:?}", e);
+                debug!("Error parsing args: {e:?}");
                 return -libc::EINVAL;
             }
         }
@@ -924,7 +1242,7 @@ pub unsafe extern "C" fn krun_set_env(ctx_id: u32, c_envp: *const *const c_char)
         match collapse_str_array(envp_array) {
             Ok(s) => s,
             Err(e) => {
-                debug!("Error parsing args: {:?}", e);
+                debug!("Error parsing args: {e:?}");
                 return -libc::EINVAL;
             }
         }
@@ -983,6 +1301,11 @@ pub unsafe extern "C" fn krun_add_vsock_port2(
     c_filepath: *const c_char,
     listen: bool,
 ) -> i32 {
+    #[cfg(feature = "nitro")]
+    if listen {
+        return -libc::EINVAL;
+    }
+
     let filepath = match CStr::from_ptr(c_filepath).to_str() {
         Ok(f) => PathBuf::from(f.to_string()),
         Err(_) => return -libc::EINVAL,
@@ -1040,6 +1363,196 @@ pub unsafe extern "C" fn krun_set_gpu_options2(
     KRUN_SUCCESS
 }
 
+#[cfg(not(feature = "gpu"))]
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+pub extern "C" fn krun_set_display_backend(
+    _ctx_id: u32,
+    _features: u32,
+    _vtable: *const c_void,
+    _vtable_size: usize,
+) -> i32 {
+    -libc::ENOTSUP
+}
+
+#[cfg(feature = "gpu")]
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+pub extern "C" fn krun_set_display_backend(
+    ctx_id: u32,
+    vtable: *const c_void,
+    vtable_size: usize,
+) -> i32 {
+    if vtable_size < size_of::<DisplayBackend>() {
+        return -libc::EINVAL;
+    }
+
+    // SAFETY: We have checked the vtable size is fine, otherwise we have to trust the user. Just
+    // to be extra careful, this uses read_unaligned, but we could probably get away with ptr::read.
+    let display_backend: DisplayBackend =
+        unsafe { std::ptr::read_unaligned(vtable as *const DisplayBackend) };
+
+    if !display_backend.verify() {
+        return -libc::EINVAL;
+    }
+
+    match CTX_MAP.lock().unwrap().entry(ctx_id) {
+        Entry::Occupied(mut ctx_cfg) => {
+            let cfg = ctx_cfg.get_mut();
+            cfg.vmr.display_backend = Some(display_backend);
+        }
+        Entry::Vacant(_) => return -libc::ENOENT,
+    }
+
+    KRUN_SUCCESS
+}
+
+#[cfg(feature = "gpu")]
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+pub unsafe extern "C" fn krun_add_display(ctx_id: u32, width: u32, height: u32) -> i32 {
+    match CTX_MAP.lock().unwrap().entry(ctx_id) {
+        Entry::Occupied(mut ctx_cfg) => {
+            let cfg = ctx_cfg.get_mut();
+            if cfg.vmr.displays.len() >= MAX_DISPLAYS {
+                return -libc::ENOMEM;
+            }
+
+            cfg.vmr.displays.push(DisplayInfo::new(width, height));
+            (cfg.vmr.displays.len() - 1) as i32
+        }
+        Entry::Vacant(_) => -libc::ENOENT,
+    }
+}
+
+#[cfg(not(feature = "gpu"))]
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+pub unsafe extern "C" fn krun_add_display(_ctx_id: u32, _width: u32, _height: u32) -> i32 {
+    -libc::ENOTSUP
+}
+
+#[cfg(feature = "gpu")]
+#[no_mangle]
+pub extern "C" fn krun_display_set_refresh_rate(
+    ctx_id: u32,
+    display_id: u32,
+    refresh_rate: u32,
+) -> i32 {
+    with_cfg(ctx_id, |cfg| {
+        let Some(display_info) = cfg.vmr.displays.get_mut(display_id as usize) else {
+            return -libc::EINVAL;
+        };
+
+        let DisplayInfoEdid::Generated(ref mut edid_params) = display_info.edid else {
+            return -libc::EALREADY;
+        };
+
+        edid_params.refresh_rate = refresh_rate;
+        KRUN_SUCCESS
+    })
+}
+
+#[cfg(not(feature = "gpu"))]
+#[no_mangle]
+pub extern "C" fn krun_display_set_refresh_rate(
+    _ctx_id: u32,
+    _display_id: u32,
+    _refresh_rate: u32,
+) -> i32 {
+    -libc::ENOTSUP
+}
+
+#[cfg(feature = "gpu")]
+#[no_mangle]
+#[allow(clippy::missing_safety_doc)]
+pub unsafe extern "C" fn krun_display_set_edid(
+    ctx_id: u32,
+    display_id: u32,
+    edid: *const u8,
+    size: size_t,
+) -> i32 {
+    with_cfg(ctx_id, |cfg| {
+        let Some(display_info) = cfg.vmr.displays.get_mut(display_id as usize) else {
+            return -libc::EINVAL;
+        };
+
+        if edid.is_null() {
+            return -libc::EINVAL;
+        }
+
+        let blob = unsafe { slice::from_raw_parts(edid, size) };
+
+        display_info.edid = DisplayInfoEdid::Provided(Box::from(blob));
+        KRUN_SUCCESS
+    })
+}
+
+#[cfg(not(feature = "gpu"))]
+#[no_mangle]
+#[allow(clippy::missing_safety_doc)]
+pub unsafe extern "C" fn krun_display_set_edid(
+    _ctx_id: u32,
+    _display_id: u32,
+    _edid: *const u8,
+    _size: size_t,
+) -> i32 {
+    -libc::ENOTSUP
+}
+
+#[cfg(feature = "gpu")]
+#[no_mangle]
+pub extern "C" fn krun_display_set_physical_size(
+    ctx_id: u32,
+    display_id: u32,
+    width_mm: u16,
+    height_mm: u16,
+) -> i32 {
+    with_cfg(ctx_id, |cfg| {
+        let Some(display_info) = cfg.vmr.displays.get_mut(display_id as usize) else {
+            return -libc::EINVAL;
+        };
+        let DisplayInfoEdid::Generated(ref mut edid_params) = display_info.edid else {
+            return -libc::EALREADY;
+        };
+        edid_params.physical_size = PhysicalSize::DimensionsMillimeters(width_mm, height_mm);
+        KRUN_SUCCESS
+    })
+}
+
+#[cfg(not(feature = "gpu"))]
+#[no_mangle]
+pub extern "C" fn krun_display_set_physical_size(
+    _ctx_id: u32,
+    _display_id: u32,
+    _width_mm: u16,
+    _height_mm: u16,
+) -> i32 {
+    -libc::ENOTSUP
+}
+
+#[cfg(feature = "gpu")]
+#[no_mangle]
+#[allow(clippy::missing_safety_doc)]
+pub extern "C" fn krun_display_set_dpi(ctx_id: u32, display_id: u32, dpi: u32) -> i32 {
+    with_cfg(ctx_id, |cfg| {
+        let Some(display_info) = cfg.vmr.displays.get_mut(display_id as usize) else {
+            return -libc::EINVAL;
+        };
+        let DisplayInfoEdid::Generated(ref mut edid_params) = display_info.edid else {
+            return -libc::EINVAL;
+        };
+        edid_params.physical_size = PhysicalSize::Dpi(dpi);
+        KRUN_SUCCESS
+    })
+}
+
+#[cfg(not(feature = "gpu"))]
+#[no_mangle]
+pub extern "C" fn krun_display_set_dpi(_ctx_id: u32, _display_id: u32, _dpi: u32) -> i32 {
+    -libc::ENOTSUP
+}
+
 #[allow(clippy::missing_safety_doc)]
 #[no_mangle]
 pub unsafe extern "C" fn krun_set_snd_device(ctx_id: u32, enable: bool) -> i32 {
@@ -1174,14 +1687,19 @@ pub unsafe extern "C" fn krun_set_smbios_oem_strings(
 }
 
 #[cfg(feature = "net")]
-fn create_virtio_net(ctx_cfg: &mut ContextConfig, backend: VirtioNetBackend) {
-    let mac = ctx_cfg.mac.unwrap_or([0x5a, 0x94, 0xef, 0xe4, 0x0c, 0xee]);
-
+fn create_virtio_net(
+    ctx_cfg: &mut ContextConfig,
+    backend: VirtioNetBackend,
+    mac: [u8; 6],
+    features: u32,
+) {
     let network_interface_config = NetworkInterfaceConfig {
-        iface_id: "eth0".to_string(),
+        iface_id: format!("eth{}", ctx_cfg.net_index),
         backend,
         mac,
+        features,
     };
+    ctx_cfg.net_index += 1;
     ctx_cfg
         .vmr
         .add_network_interface(network_interface_config)
@@ -1256,7 +1774,7 @@ pub unsafe extern "C" fn krun_set_kernel(
     let path = match CStr::from_ptr(c_kernel_path).to_str() {
         Ok(path) => PathBuf::from(path),
         Err(e) => {
-            error!("Error parsing kernel_path: {:?}", e);
+            error!("Error parsing kernel_path: {e:?}");
             return -libc::EINVAL;
         }
     };
@@ -1285,14 +1803,14 @@ pub unsafe extern "C" fn krun_set_kernel(
                 let size = match std::fs::metadata(&path) {
                     Ok(metadata) => metadata.len(),
                     Err(e) => {
-                        error!("Can't read initramfs metadata: {:?}", e);
+                        error!("Can't read initramfs metadata: {e:?}");
                         return -libc::EINVAL;
                     }
                 };
                 (Some(path), size)
             }
             Err(e) => {
-                error!("Error parsing initramfs path: {:?}", e);
+                error!("Error parsing initramfs path: {e:?}");
                 return -libc::EINVAL;
             }
         }
@@ -1304,7 +1822,7 @@ pub unsafe extern "C" fn krun_set_kernel(
         match CStr::from_ptr(c_cmdline).to_str() {
             Ok(cmdline) => Some(cmdline.to_string()),
             Err(e) => {
-                error!("Error parsing kernel cmdline: {:?}", e);
+                error!("Error parsing kernel cmdline: {e:?}");
                 return -libc::EINVAL;
             }
         }
@@ -1399,7 +1917,140 @@ pub extern "C" fn krun_setgid(ctx_id: u32, gid: libc::gid_t) -> i32 {
     KRUN_SUCCESS
 }
 
+#[cfg(feature = "nitro")]
+#[allow(clippy::missing_safety_doc)]
 #[no_mangle]
+pub unsafe extern "C" fn krun_nitro_set_image(ctx_id: u32, c_image_filepath: *const c_char) -> i32 {
+    let filepath = match CStr::from_ptr(c_image_filepath).to_str() {
+        Ok(f) => PathBuf::from(f.to_string()),
+        Err(_) => return -libc::EINVAL,
+    };
+
+    match CTX_MAP.lock().unwrap().entry(ctx_id) {
+        Entry::Occupied(mut ctx_cfg) => {
+            let cfg = ctx_cfg.get_mut();
+            cfg.set_nitro_image(filepath);
+        }
+        Entry::Vacant(_) => return -libc::ENOENT,
+    }
+
+    KRUN_SUCCESS
+}
+
+#[cfg(feature = "nitro")]
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+pub unsafe extern "C" fn krun_nitro_set_start_flags(ctx_id: u32, start_flags: u64) -> i32 {
+    let mut flags = StartFlags::empty();
+
+    // Only debug mode is supported at the moment. To avoid doing conversion and
+    // checking if the "start_flags" argument is valid, set the flags to debug mode
+    // if the "start_flags" argument is greater than zero.
+    if start_flags > 0 {
+        flags |= StartFlags::DEBUG;
+    }
+
+    match CTX_MAP.lock().unwrap().entry(ctx_id) {
+        Entry::Occupied(mut ctx_cfg) => {
+            let cfg = ctx_cfg.get_mut();
+            cfg.set_nitro_start_flags(flags);
+        }
+        Entry::Vacant(_) => return -libc::ENOENT,
+    }
+
+    KRUN_SUCCESS
+}
+
+#[cfg(all(feature = "blk", not(feature = "tee")))]
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+pub unsafe extern "C" fn krun_set_root_disk_remount(
+    ctx_id: u32,
+    c_device: *const c_char,
+    c_fstype: *const c_char,
+    c_options: *const c_char,
+) -> i32 {
+    let device = match CStr::from_ptr(c_device).to_str() {
+        Ok(device) => device.to_string(),
+        Err(e) => {
+            error!("Error parsing device path: {e:?}");
+            return -libc::EINVAL;
+        }
+    };
+
+    let fstype = if !c_fstype.is_null() {
+        match CStr::from_ptr(c_fstype).to_str() {
+            Ok(fstype) => {
+                if fstype == "auto" {
+                    None
+                } else {
+                    Some(fstype.to_string())
+                }
+            }
+            Err(e) => {
+                error!("Error parsing fstype: {e:?}");
+                return -libc::EINVAL;
+            }
+        }
+    } else {
+        None
+    };
+
+    let options = if !c_options.is_null() {
+        match CStr::from_ptr(c_options).to_str() {
+            Ok(options) => Some(options.to_string()),
+            Err(e) => {
+                error!("Error parsing options: {e:?}");
+                return -libc::EINVAL;
+            }
+        }
+    } else {
+        None
+    };
+
+    match CTX_MAP.lock().unwrap().entry(ctx_id) {
+        Entry::Occupied(mut ctx_cfg) => {
+            let ctx_cfg = ctx_cfg.get_mut();
+
+            if ctx_cfg.vmr.fs.iter().any(|fs| fs.fs_id == "/dev/root") {
+                error!("Root filesystem already configured");
+                return -libc::EINVAL;
+            }
+
+            if ctx_cfg.block_cfgs.is_empty() {
+                error!("No block devices configured");
+                return -libc::EINVAL;
+            }
+
+            // To boot from a filesystem other than virtiofs,
+            // we need to setup a temporary root from which init.krun can be executed.
+            // Otherwise, it would have to be copied to the target filesystem beforehand.
+            // Instead, init.krun will run from virtiofs and then switch to the real root.
+            let root_dir_suffix = Alphanumeric.sample_string(&mut rand::rng(), 6);
+            let empty_root = env::temp_dir().join(format!("krun-empty-root-{root_dir_suffix}"));
+
+            if let Err(e) = std::fs::create_dir_all(&empty_root) {
+                error!("Failed to create empty root directory: {e:?}");
+                return -libc::EINVAL;
+            }
+
+            ctx_cfg.vmr.add_fs_device(FsDeviceConfig {
+                fs_id: "/dev/root".into(),
+                shared_dir: empty_root.to_string_lossy().into(),
+                // Default to a conservative 512 MB window.
+                shm_size: Some(1 << 29),
+            });
+
+            ctx_cfg.set_block_root(device, fstype, options);
+        }
+        Entry::Vacant(_) => return -libc::ENOENT,
+    };
+
+    KRUN_SUCCESS
+}
+
+#[no_mangle]
+#[allow(unreachable_code)]
 pub extern "C" fn krun_start_enter(ctx_id: u32) -> i32 {
     #[cfg(target_os = "linux")]
     {
@@ -1410,10 +2061,13 @@ pub extern "C" fn krun_start_enter(ctx_id: u32) -> i32 {
         unsafe { libc::prctl(libc::PR_SET_NAME, prname.as_ptr()) };
     }
 
+    #[cfg(feature = "nitro")]
+    return krun_start_enter_nitro(ctx_id);
+
     let mut event_manager = match EventManager::new() {
         Ok(em) => em,
         Err(e) => {
-            error!("Unable to create EventManager: {:?}", e);
+            error!("Unable to create EventManager: {e:?}");
             return -libc::EINVAL;
         }
     };
@@ -1431,7 +2085,7 @@ pub extern "C" fn krun_start_enter(ctx_id: u32) -> i32 {
                 return -libc::ENOENT;
             }
         } else {
-            eprintln!("Couldn't find or load {}", KRUNFW_NAME);
+            eprintln!("Couldn't find or load {KRUNFW_NAME}");
             return -libc::ENOENT;
         }
     }
@@ -1453,7 +2107,7 @@ pub extern "C" fn krun_start_enter(ctx_id: u32) -> i32 {
     #[cfg(feature = "tee")]
     if let Some(tee_config) = ctx_cfg.get_tee_config_file() {
         if let Err(e) = ctx_cfg.vmr.set_tee_config(tee_config) {
-            error!("Error setting up TEE config: {:?}", e);
+            error!("Error setting up TEE config: {e:?}");
             return -libc::EINVAL;
         }
     } else {
@@ -1462,12 +2116,12 @@ pub extern "C" fn krun_start_enter(ctx_id: u32) -> i32 {
     }
 
     let boot_source = BootSourceConfig {
-        kernel_cmdline_prolog: Some(format!(
-            "{} init={} {} {} {} {}",
-            DEFAULT_KERNEL_CMDLINE,
-            INIT_PATH,
+        kernel_cmdline_prolog: Some(format!("{} init={}", DEFAULT_KERNEL_CMDLINE, INIT_PATH)),
+        kernel_cmdline_krun_env: Some(format!(
+            " {} {} {} {} {}",
             ctx_cfg.get_exec_path(),
             ctx_cfg.get_workdir(),
+            ctx_cfg.get_block_root(),
             ctx_cfg.get_rlimits(),
             ctx_cfg.get_env(),
         )),
@@ -1478,6 +2132,23 @@ pub extern "C" fn krun_start_enter(ctx_id: u32) -> i32 {
         return -libc::EINVAL;
     }
 
+    #[cfg(feature = "net")]
+    {
+        if let Some(legacy_net_cfg) = ctx_cfg.legacy_net_cfg.clone() {
+            let backend = match legacy_net_cfg {
+                LegacyNetworkConfig::VirtioNetGvproxy(path) => {
+                    VirtioNetBackend::UnixgramPath(path, true)
+                }
+                LegacyNetworkConfig::VirtioNetPasst(fd) => VirtioNetBackend::UnixstreamFd(fd),
+            };
+            let mac = ctx_cfg
+                .legacy_mac
+                .unwrap_or([0x5a, 0x94, 0xef, 0xe4, 0x0c, 0xee]);
+            create_virtio_net(&mut ctx_cfg, backend, mac, NET_COMPAT_FEATURES);
+        }
+    }
+
+    #[allow(unused_assignments)]
     let mut vsock_set = false;
     let mut vsock_config = VsockDeviceConfig {
         vsock_id: "vsock0".to_string(),
@@ -1486,30 +2157,20 @@ pub extern "C" fn krun_start_enter(ctx_id: u32) -> i32 {
         unix_ipc_port_map: None,
     };
 
-    if let Some(ref map) = ctx_cfg.unix_ipc_port_map {
-        vsock_config.unix_ipc_port_map = Some(map.clone());
+    #[cfg(feature = "net")]
+    if ctx_cfg.vmr.net.list.is_empty() && ctx_cfg.legacy_net_cfg.is_none() {
+        vsock_config.host_port_map = ctx_cfg.tsi_port_map;
+        vsock_set = true;
+    }
+    #[cfg(not(feature = "net"))]
+    {
+        vsock_config.host_port_map = ctx_cfg.tsi_port_map;
         vsock_set = true;
     }
 
-    match ctx_cfg.net_cfg {
-        NetworkConfig::Tsi(tsi_cfg) => {
-            vsock_config.host_port_map = tsi_cfg.port_map;
-            vsock_set = true;
-        }
-        NetworkConfig::VirtioNetPasst(_fd) => {
-            #[cfg(feature = "net")]
-            {
-                let backend = VirtioNetBackend::Passt(_fd);
-                create_virtio_net(&mut ctx_cfg, backend);
-            }
-        }
-        NetworkConfig::VirtioNetGvproxy(ref _path) => {
-            #[cfg(feature = "net")]
-            {
-                let backend = VirtioNetBackend::Gvproxy(_path.clone());
-                create_virtio_net(&mut ctx_cfg, backend);
-            }
-        }
+    if let Some(ref map) = ctx_cfg.unix_ipc_port_map {
+        vsock_config.unix_ipc_port_map = Some(map.clone());
+        vsock_set = true;
     }
 
     if vsock_set {
@@ -1532,14 +2193,14 @@ pub extern "C" fn krun_start_enter(ctx_id: u32) -> i32 {
 
     if let Some(gid) = ctx_cfg.vmm_gid {
         if unsafe { libc::setgid(gid) } != 0 {
-            error!("Failed to set gid {}", gid);
+            error!("Failed to set gid {gid}");
             return -std::io::Error::last_os_error().raw_os_error().unwrap();
         }
     }
 
     if let Some(uid) = ctx_cfg.vmm_uid {
         if unsafe { libc::setuid(uid) } != 0 {
-            error!("Failed to set uid {}", uid);
+            error!("Failed to set uid {uid}");
             return -std::io::Error::last_os_error().raw_os_error().unwrap();
         }
     }
@@ -1554,7 +2215,7 @@ pub extern "C" fn krun_start_enter(ctx_id: u32) -> i32 {
     ) {
         Ok(vmm) => vmm,
         Err(e) => {
-            error!("Building the microVM failed: {:?}", e);
+            error!("Building the microVM failed: {e:?}");
             return -libc::EINVAL;
         }
     };
@@ -1569,16 +2230,39 @@ pub extern "C" fn krun_start_enter(ctx_id: u32) -> i32 {
         vmm::worker::start_worker_thread(_vmm.clone(), _receiver.clone()).unwrap();
     }
 
-    #[cfg(feature = "amd-sev")]
+    #[cfg(any(feature = "amd-sev", feature = "tdx"))]
     vmm::worker::start_worker_thread(_vmm.clone(), _receiver.clone()).unwrap();
 
     loop {
         match event_manager.run() {
             Ok(_) => {}
             Err(e) => {
-                error!("Error in EventManager loop: {:?}", e);
+                error!("Error in EventManager loop: {e:?}");
                 return -libc::EINVAL;
             }
         }
     }
 }
+
+#[cfg(feature = "nitro")]
+#[no_mangle]
+fn krun_start_enter_nitro(ctx_id: u32) -> i32 {
+    let ctx_cfg = match CTX_MAP.lock().unwrap().remove(&ctx_id) {
+        Some(ctx_cfg) => ctx_cfg,
+        None => return -libc::ENOENT,
+    };
+
+    let Ok(mut enclave) = NitroEnclave::try_from(ctx_cfg) else {
+        return -libc::EINVAL;
+    };
+
+    // Return enclave CID if successfully ran.
+    match enclave.run() {
+        Ok(cid) => cid.try_into().unwrap(), // Safe to unwrap.
+        Err(e) => {
+            error!("Error running nitro enclave: {e}");
+
+            -libc::EINVAL
+        }
+    }
+}
