@@ -25,6 +25,10 @@ use crate::device_manager::legacy::PortIODeviceManager;
 use crate::device_manager::mmio::MMIODeviceManager;
 use crate::resources::VmResources;
 use crate::vmm_config::external_kernel::{ExternalKernel, KernelFormat};
+#[cfg(feature = "net")]
+use crate::vmm_config::net::NetBuilder;
+#[cfg(all(target_os = "linux", target_arch = "riscv64"))]
+use devices::legacy::KvmAia;
 #[cfg(all(target_os = "linux", target_arch = "aarch64"))]
 use devices::legacy::KvmGicV3;
 #[cfg(target_arch = "x86_64")]
@@ -37,16 +41,12 @@ use devices::legacy::{GicV3, HvfGicV3};
 #[cfg(target_arch = "x86_64")]
 use devices::legacy::{IoApic, IrqChipT};
 use devices::legacy::{IrqChip, IrqChipDevice};
-#[cfg(feature = "net")]
-use devices::virtio::Net;
-use devices::virtio::{port_io, MmioTransport, PortDescription, Vsock};
+use devices::virtio::{port_io, MmioTransport, PortDescription, VirtioDevice, Vsock};
 
 #[cfg(feature = "tee")]
 use kbs_types::Tee;
 
 use crate::device_manager;
-#[cfg(feature = "tee")]
-use crate::resources::TeeConfig;
 #[cfg(target_os = "linux")]
 use crate::signal_handler::register_sigint_handler;
 #[cfg(target_os = "linux")]
@@ -55,7 +55,7 @@ use crate::terminal::term_set_raw_mode;
 #[cfg(feature = "blk")]
 use crate::vmm_config::block::BlockBuilder;
 use crate::vmm_config::boot_source::DEFAULT_KERNEL_CMDLINE;
-#[cfg(not(feature = "tee"))]
+#[cfg(not(any(feature = "tee", feature = "nitro")))]
 use crate::vmm_config::fs::FsDeviceConfig;
 #[cfg(target_os = "linux")]
 use crate::vstate::KvmContext;
@@ -64,10 +64,18 @@ use crate::vstate::MeasuredRegion;
 use crate::vstate::{Error as VstateError, Vcpu, VcpuConfig, Vm};
 use arch::{ArchMemoryInfo, InitrdConfig};
 use device_manager::shm::ShmManager;
-#[cfg(not(feature = "tee"))]
+#[cfg(feature = "gpu")]
+use devices::virtio::display::DisplayInfo;
+#[cfg(feature = "gpu")]
+use devices::virtio::display::NoopDisplayBackend;
+#[cfg(not(any(feature = "tee", feature = "nitro")))]
 use devices::virtio::{fs::ExportTable, VirtioShmRegion};
 use flate2::read::GzDecoder;
-#[cfg(feature = "tee")]
+#[cfg(feature = "gpu")]
+use krun_display::DisplayBackend;
+#[cfg(feature = "gpu")]
+use krun_display::IntoDisplayBackend;
+#[cfg(feature = "amd-sev")]
 use kvm_bindings::KVM_MAX_CPUID_ENTRIES;
 use libc::{STDERR_FILENO, STDIN_FILENO, STDOUT_FILENO};
 #[cfg(target_arch = "x86_64")]
@@ -78,12 +86,14 @@ use utils::eventfd::EventFd;
 use utils::worker_message::WorkerMessage;
 #[cfg(all(target_arch = "x86_64", not(feature = "efi"), not(feature = "tee")))]
 use vm_memory::mmap::MmapRegion;
-#[cfg(not(feature = "tee"))]
+#[cfg(not(any(feature = "tee", feature = "nitro")))]
 use vm_memory::Address;
 use vm_memory::Bytes;
+#[cfg(not(feature = "nitro"))]
+use vm_memory::GuestMemory;
 #[cfg(all(target_arch = "x86_64", not(feature = "tee")))]
 use vm_memory::GuestRegionMmap;
-use vm_memory::{GuestAddress, GuestMemory, GuestMemoryMmap};
+use vm_memory::{GuestAddress, GuestMemoryMmap};
 
 #[cfg(feature = "efi")]
 static EDK2_BINARY: &[u8] = include_bytes!("../../../edk2/KRUN_EFI.silent.fd");
@@ -173,6 +183,8 @@ pub enum StartMicrovmError {
     RegisterEvent(EventManagerError),
     /// Cannot initialize a MMIO Fs Device or add ad device to the MMIO Bus.
     RegisterFsDevice(device_manager::mmio::Error),
+    // Cannot initialize a MMIO Fs Device or add ad device to the MMIO Bus.
+    RegisterConsoleDevice(device_manager::mmio::Error),
     /// Cannot register SIGWINCH event file descriptor.
     #[cfg(target_os = "linux")]
     RegisterFsSigwinch(kvm_ioctls::Error),
@@ -363,6 +375,15 @@ impl Display for StartMicrovmError {
                     "Cannot initialize a MMIO Fs Device or add a device to the MMIO Bus. {err_msg}"
                 )
             }
+            RegisterConsoleDevice(ref err) => {
+                let mut err_msg = format!("{err}");
+                err_msg = err_msg.replace('\"', "");
+
+                write!(
+                    f,
+                    "Cannot initialize a MMIO Console Device or add a device to the MMIO Bus. {err_msg}"
+                )
+            }
             #[cfg(target_os = "linux")]
             RegisterFsSigwinch(ref err) => {
                 let mut err_msg = format!("{err}");
@@ -434,7 +455,7 @@ impl Display for StartMicrovmError {
                 )
             }
             ShmHostAddr(ref err) => {
-                let mut err_msg = format!("{:?}", err);
+                let mut err_msg = format!("{err:?}");
                 err_msg = err_msg.replace('\"', "");
 
                 write!(
@@ -443,13 +464,13 @@ impl Display for StartMicrovmError {
                 )
             }
             ShmConfig(ref err) => {
-                let mut err_msg = format!("{:?}", err);
+                let mut err_msg = format!("{err:?}");
                 err_msg = err_msg.replace('\"', "");
 
                 write!(f, "Error while configuring an SHM region. {err_msg}")
             }
             ShmCreate(ref err) => {
-                let mut err_msg = format!("{:?}", err);
+                let mut err_msg = format!("{err:?}");
                 err_msg = err_msg.replace('\"', "");
 
                 write!(f, "Error while creating an SHM region. {err_msg}")
@@ -464,7 +485,7 @@ impl Display for StartMicrovmError {
 enum Payload {
     #[cfg(all(target_arch = "x86_64", not(feature = "tee")))]
     KernelMmap,
-    #[cfg(target_arch = "aarch64")]
+    #[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
     KernelCopy,
     ExternalKernel(ExternalKernel),
     #[cfg(test)]
@@ -487,7 +508,7 @@ fn choose_payload(vm_resources: &VmResources) -> Result<Payload, StartMicrovmErr
         #[cfg(all(target_os = "linux", target_arch = "x86_64", not(feature = "tee")))]
         return Ok(Payload::KernelMmap);
 
-        #[cfg(target_arch = "aarch64")]
+        #[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
         return Ok(Payload::KernelCopy);
     } else if let Some(external_kernel) = vm_resources.external_kernel() {
         Ok(Payload::ExternalKernel(external_kernel.clone()))
@@ -534,23 +555,33 @@ pub fn build_microvm(
         kernel_cmdline.insert_str(DEFAULT_KERNEL_CMDLINE).unwrap();
     }
 
+    if let Some(cmdline) = &vm_resources.boot_config.kernel_cmdline_krun_env {
+        kernel_cmdline.insert_str(cmdline.as_str()).unwrap();
+    }
+
     #[cfg(not(feature = "tee"))]
     #[allow(unused_mut)]
     let mut vm = setup_vm(&guest_memory, vm_resources.nested_enabled)?;
 
     #[cfg(feature = "tee")]
-    let (kvm, vm) = {
+    let (_kvm, vm) = {
         let kvm = KvmContext::new()
             .map_err(Error::KvmContext)
             .map_err(StartMicrovmError::Internal)?;
-        let vm = setup_vm(&kvm, &guest_memory, vm_resources.tee_config())?;
+        let vm = setup_vm(
+            &kvm,
+            &guest_memory,
+            vm_resources,
+            #[cfg(feature = "tdx")]
+            _sender.clone(),
+        )?;
         (kvm, vm)
     };
 
     #[cfg(feature = "tee")]
     let tee = vm_resources.tee_config().tee;
 
-    #[cfg(feature = "tee")]
+    #[cfg(feature = "amd-sev")]
     let snp_launcher = match tee {
         Tee::Snp => Some(
             vm.snp_secure_virt_prepare(&guest_memory)
@@ -559,7 +590,15 @@ pub fn build_microvm(
         _ => None,
     };
 
-    #[cfg(feature = "tee")]
+    #[cfg(feature = "tdx")]
+    let mut tdx_launcher = match tee {
+        Tee::Tdx => vm
+            .tdx_secure_virt_prepare()
+            .map_err(StartMicrovmError::SecureVirtPrepare)?,
+        _ => panic!(),
+    };
+
+    #[cfg(all(feature = "tee", not(feature = "tdx")))]
     let measured_regions = {
         println!("Injecting and measuring memory regions. This may take a while.");
 
@@ -611,6 +650,32 @@ pub fn build_microvm(
         ]
     };
 
+    #[cfg(feature = "tdx")]
+    let measured_regions = {
+        println!("Injecting and measuring memory regions. This may take a while.");
+        let qboot_size = if let Some(qboot_bundle) = &vm_resources.qboot_bundle {
+            qboot_bundle.size
+        } else {
+            return Err(StartMicrovmError::MissingKernelConfig);
+        };
+        let m = vec![
+            MeasuredRegion {
+                guest_addr: 0,
+                host_addr: guest_memory.get_host_address(GuestAddress(0)).unwrap() as u64,
+                size: 0x8000_0000,
+            },
+            MeasuredRegion {
+                guest_addr: arch::BIOS_START,
+                host_addr: guest_memory
+                    .get_host_address(GuestAddress(arch::BIOS_START))
+                    .unwrap() as u64,
+                size: qboot_size,
+            },
+        ];
+
+        m
+    };
+
     // On x86_64 always create a serial device,
     // while on aarch64 only create it if 'console=' is specified in the boot args.
     let serial_device = if cfg!(feature = "efi") {
@@ -694,6 +759,14 @@ pub fn build_microvm(
         .map_err(StartMicrovmError::Internal)?;
     }
 
+    #[cfg(feature = "tdx")]
+    {
+        for vcpu in &vcpus {
+            vcpu.tdx_secure_virt_prepare(&mut tdx_launcher);
+        }
+        vm.tdx_secure_virt_init_vcpus(&mut tdx_launcher).unwrap();
+    }
+
     // On aarch64, the vCPUs need to be created (i.e call KVM_CREATE_VCPU) and configured before
     // setting up the IRQ chip because the `KVM_CREATE_VCPU` ioctl will return error if the IRQCHIP
     // was already initialized.
@@ -756,6 +829,29 @@ pub fn build_microvm(
         )?;
     }
 
+    #[cfg(all(target_arch = "riscv64", target_os = "linux"))]
+    {
+        vcpus = create_vcpus_riscv64(
+            &vm,
+            &vcpu_config,
+            &guest_memory,
+            payload_config.entry_addr,
+            &exit_evt,
+        )
+        .map_err(StartMicrovmError::Internal)?;
+
+        intc = Arc::new(Mutex::new(IrqChipDevice::new(Box::new(
+            KvmAia::new(vm.fd(), vm_resources.vm_config().vcpu_count.unwrap() as u32).unwrap(),
+        ))));
+
+        attach_legacy_devices(
+            &vm,
+            &mut mmio_device_manager,
+            &mut kernel_cmdline,
+            serial_device,
+        )?;
+    }
+
     // We use this atomic to record the exit code set by init/init.c in the VM.
     let exit_code = Arc::new(AtomicI32::new(i32::MAX));
 
@@ -784,7 +880,7 @@ pub fn build_microvm(
         vm_resources.console_output.clone(),
     )?;
 
-    #[cfg(not(feature = "tee"))]
+    #[cfg(not(any(feature = "tee", feature = "nitro")))]
     let export_table: Option<ExportTable> = if cfg!(feature = "gpu") {
         Some(Default::default())
     } else {
@@ -793,6 +889,10 @@ pub fn build_microvm(
 
     #[cfg(feature = "gpu")]
     if let Some(virgl_flags) = vm_resources.gpu_virgl_flags {
+        let display_backend = vm_resources
+            .display_backend
+            .unwrap_or_else(|| NoopDisplayBackend::into_display_backend(None));
+
         attach_gpu_device(
             &mut vmm,
             event_manager,
@@ -801,12 +901,13 @@ pub fn build_microvm(
             export_table.clone(),
             intc.clone(),
             virgl_flags,
+            Box::from(&vm_resources.displays[..]),
+            display_backend,
             #[cfg(target_os = "macos")]
             _sender.clone(),
         )?;
     }
-
-    #[cfg(not(feature = "tee"))]
+    #[cfg(not(any(feature = "tee", feature = "nitro")))]
     attach_fs_devices(
         &mut vmm,
         &vm_resources.fs,
@@ -825,18 +926,13 @@ pub fn build_microvm(
         #[cfg(not(feature = "net"))]
         vmm.kernel_cmdline.insert_str("tsi_hijack")?;
         #[cfg(feature = "net")]
-        if vm_resources
-            .net_builder
-            .iter()
-            .collect::<Vec<_>>()
-            .is_empty()
-        {
+        if vm_resources.net.list.is_empty() {
             // Only enable TSI if we don't have any network devices.
             vmm.kernel_cmdline.insert_str("tsi_hijack")?;
         }
     }
     #[cfg(feature = "net")]
-    attach_net_devices(&mut vmm, vm_resources.net_builder.iter(), intc.clone())?;
+    attach_net_devices(&mut vmm, &vm_resources.net, intc.clone())?;
     #[cfg(feature = "snd")]
     if vm_resources.snd_device {
         attach_snd_device(&mut vmm, intc.clone())?;
@@ -862,8 +958,9 @@ pub fn build_microvm(
     #[cfg(feature = "tee")]
     {
         match tee {
+            #[cfg(feature = "amd-sev")]
             Tee::Snp => {
-                let cpuid = kvm
+                let cpuid = _kvm
                     .fd()
                     .get_supported_cpuid(KVM_MAX_CPUID_ENTRIES)
                     .map_err(VstateError::KvmCpuId)
@@ -877,6 +974,15 @@ pub fn build_microvm(
                     )
                     .map_err(StartMicrovmError::SecureVirtAttest)?;
             }
+            #[cfg(feature = "tdx")]
+            Tee::Tdx => {
+                vmm.kvm_vm()
+                    .tdx_secure_virt_prepare_memory(&mut tdx_launcher, &measured_regions)
+                    .unwrap();
+                vmm.kvm_vm()
+                    .tdx_secure_virt_finalize_vm(tdx_launcher)
+                    .map_err(StartMicrovmError::SecureVirtPrepare)?;
+            }
             _ => return Err(StartMicrovmError::InvalidTee),
         }
 
@@ -906,7 +1012,7 @@ fn load_external_kernel(
         // Raw images are treated as bundled kernels on x86_64
         #[cfg(target_arch = "x86_64")]
         KernelFormat::Raw => unreachable!(),
-        #[cfg(target_arch = "aarch64")]
+        #[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
         KernelFormat::Raw => {
             let data: Vec<u8> = std::fs::read(external_kernel.path.clone())
                 .map_err(StartMicrovmError::RawOpenKernel)?;
@@ -924,7 +1030,7 @@ fn load_external_kernel(
                 .map_err(StartMicrovmError::ElfLoadKernel)?;
             load_result.kernel_load
         }
-        #[cfg(target_arch = "aarch64")]
+        #[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
         KernelFormat::PeGz => {
             let data: Vec<u8> = std::fs::read(external_kernel.path.clone())
                 .map_err(StartMicrovmError::PeGzOpenKernel)?;
@@ -932,7 +1038,7 @@ fn load_external_kernel(
                 .windows(3)
                 .position(|window| window == [0x1f, 0x8b, 0x8])
             {
-                debug!("Found GZIP header on PE file at: 0x{:x}", magic);
+                debug!("Found GZIP header on PE file at: 0x{magic:x}");
                 let (_, compressed) = data.split_at(magic);
                 let mut gz = GzDecoder::new(compressed);
                 let mut kernel_data: Vec<u8> = Vec::new();
@@ -954,7 +1060,7 @@ fn load_external_kernel(
                 .windows(4)
                 .position(|window| window == [b'B', b'Z', b'h'])
             {
-                debug!("Found BZIP2 header on Image file at: 0x{:x}", magic);
+                debug!("Found BZIP2 header on Image file at: 0x{magic:x}");
                 let (_, compressed) = data.split_at(magic);
                 let mut kernel_data: Vec<u8> = Vec::new();
                 let mut bz2 = bzip2::read::BzDecoder::new(compressed);
@@ -980,7 +1086,7 @@ fn load_external_kernel(
                 .windows(3)
                 .position(|window| window == [0x1f, 0x8b, 0x8])
             {
-                debug!("Found GZIP header on Image file at: 0x{:x}", magic);
+                debug!("Found GZIP header on Image file at: 0x{magic:x}");
                 let (_, compressed) = data.split_at(magic);
                 let mut gz = GzDecoder::new(compressed);
                 let mut kernel_data: Vec<u8> = Vec::new();
@@ -1006,7 +1112,7 @@ fn load_external_kernel(
                 .windows(4)
                 .position(|window| window == [0x28, 0xb5, 0x2f, 0xfd])
             {
-                debug!("Found ZSTD header on Image file at: 0x{:x}", magic);
+                debug!("Found ZSTD header on Image file at: 0x{magic:x}");
                 let (_, zstd_data) = data.split_at(magic);
                 let mut kernel_data: Vec<u8> = Vec::new();
                 let _ = zstd::stream::copy_decode(zstd_data, &mut kernel_data);
@@ -1058,7 +1164,7 @@ fn load_payload(
     StartMicrovmError,
 > {
     match payload {
-        #[cfg(target_arch = "aarch64")]
+        #[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
         Payload::KernelCopy => {
             let (kernel_entry_addr, kernel_host_addr, kernel_guest_addr, kernel_size) =
                 if let Some(kernel_bundle) = &_vm_resources.kernel_bundle {
@@ -1228,7 +1334,7 @@ fn create_guest_memory(
         Payload::Empty => arch::arch_memory_regions(mem_size, None, 0, 0),
         Payload::Efi => unreachable!(),
     };
-    #[cfg(target_arch = "aarch64")]
+    #[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
     let (arch_mem_info, mut arch_mem_regions) = match payload {
         Payload::ExternalKernel(external_kernel) => {
             arch::arch_memory_regions(mem_size, external_kernel.initramfs_size)
@@ -1302,11 +1408,17 @@ pub(crate) fn setup_vm(
 pub(crate) fn setup_vm(
     kvm: &KvmContext,
     guest_memory: &GuestMemoryMmap,
-    tee_config: &TeeConfig,
+    resources: &super::resources::VmResources,
+    #[cfg(feature = "tdx")] _sender: Sender<WorkerMessage>,
 ) -> std::result::Result<Vm, StartMicrovmError> {
-    let mut vm = Vm::new(kvm.fd(), tee_config)
-        .map_err(Error::Vm)
-        .map_err(StartMicrovmError::Internal)?;
+    let mut vm = Vm::new(
+        kvm.fd(),
+        resources.tee_config(),
+        #[cfg(feature = "tdx")]
+        _sender,
+    )
+    .map_err(Error::Vm)
+    .map_err(StartMicrovmError::Internal)?;
     vm.memory_init(guest_memory, kvm.max_memslots())
         .map_err(Error::Vm)
         .map_err(StartMicrovmError::Internal)?;
@@ -1344,7 +1456,7 @@ pub fn setup_serial_device(
             // has been redirected to /dev/null via dup2 (this may happen inside the jailer).
             // Find a better solution to this (and think about the state of the serial device
             // while we're at it).
-            warn!("Could not add serial input event to epoll: {:?}", e);
+            warn!("Could not add serial input event to epoll: {e:?}");
         }
     }
     Ok(serial)
@@ -1389,7 +1501,10 @@ fn attach_legacy_devices(
     Ok(())
 }
 
-#[cfg(all(target_arch = "aarch64", target_os = "linux"))]
+#[cfg(all(
+    any(target_arch = "aarch64", target_arch = "riscv64"),
+    target_os = "linux"
+))]
 fn attach_legacy_devices(
     vm: &Vm,
     mmio_device_manager: &mut MMIODeviceManager,
@@ -1403,6 +1518,7 @@ fn attach_legacy_devices(
             .map_err(StartMicrovmError::Internal)?;
     }
 
+    #[cfg(all(target_arch = "aarch64", target_os = "linux"))]
     mmio_device_manager
         .register_mmio_rtc(vm.fd())
         .map_err(Error::RegisterMMIODevice)
@@ -1550,27 +1666,51 @@ fn create_vcpus_aarch64(
     Ok(vcpus)
 }
 
-/// Attaches an MmioTransport device to the device manager.
+#[cfg(all(target_arch = "riscv64", target_os = "linux"))]
+fn create_vcpus_riscv64(
+    vm: &Vm,
+    vcpu_config: &VcpuConfig,
+    guest_mem: &GuestMemoryMmap,
+    entry_addr: GuestAddress,
+    exit_evt: &EventFd,
+) -> super::Result<Vec<Vcpu>> {
+    let mut vcpus = Vec::with_capacity(vcpu_config.vcpu_count as usize);
+    for cpu_index in 0..vcpu_config.vcpu_count {
+        let mut vcpu = Vcpu::new_riscv64(
+            cpu_index,
+            vm.fd(),
+            exit_evt.try_clone().map_err(Error::EventFd)?,
+        )
+        .map_err(Error::Vcpu)?;
+
+        vcpu.configure_riscv64(vm.fd(), guest_mem, entry_addr)
+            .map_err(Error::Vcpu)?;
+
+        vcpus.push(vcpu);
+    }
+    Ok(vcpus)
+}
+
+/// Attaches an virtio mmio device to the device manager.
 fn attach_mmio_device(
     vmm: &mut Vmm,
     id: String,
-    device: MmioTransport,
+    intc: IrqChip,
+    device: Arc<Mutex<dyn VirtioDevice>>,
 ) -> std::result::Result<(), device_manager::mmio::Error> {
-    let type_id = device
-        .device()
-        .lock()
-        .expect("Poisoned device lock")
-        .device_type();
+    let mmio_device = MmioTransport::new(vmm.guest_memory().clone(), intc, device)?;
+
+    let type_id = mmio_device.locked_device().device_type();
     let _cmdline = &mut vmm.kernel_cmdline;
 
     #[cfg(target_os = "linux")]
     let (_mmio_base, _irq) =
         vmm.mmio_device_manager
-            .register_mmio_device(vmm.vm.fd(), device, type_id, id)?;
+            .register_mmio_device(vmm.vm.fd(), mmio_device, type_id, id)?;
     #[cfg(target_os = "macos")]
-    let (_mmio_base, _irq) = vmm
-        .mmio_device_manager
-        .register_mmio_device(device, type_id, id)?;
+    let (_mmio_base, _irq) =
+        vmm.mmio_device_manager
+            .register_mmio_device(mmio_device, type_id, id)?;
 
     #[cfg(target_arch = "x86_64")]
     vmm.mmio_device_manager
@@ -1579,7 +1719,7 @@ fn attach_mmio_device(
     Ok(())
 }
 
-#[cfg(not(feature = "tee"))]
+#[cfg(not(any(feature = "tee", feature = "nitro")))]
 fn attach_fs_devices(
     vmm: &mut Vmm,
     fs_devs: &[FsDeviceConfig],
@@ -1603,8 +1743,6 @@ fn attach_fs_devices(
 
         let id = format!("{}{}", String::from(fs.lock().unwrap().id()), i);
 
-        fs.lock().unwrap().set_intc(intc.clone());
-
         if let Some(shm_region) = shm_manager.fs_region(i) {
             fs.lock().unwrap().set_shm_region(VirtioShmRegion {
                 host_addr: vmm
@@ -1625,12 +1763,7 @@ fn attach_fs_devices(
         fs.lock().unwrap().set_map_sender(map_sender.clone());
 
         // The device mutex mustn't be locked here otherwise it will deadlock.
-        attach_mmio_device(
-            vmm,
-            id,
-            MmioTransport::new(vmm.guest_memory().clone(), fs.clone()),
-        )
-        .map_err(RegisterFsDevice)?;
+        attach_mmio_device(vmm, id, intc.clone(), fs).map_err(RegisterFsDevice)?;
     }
 
     Ok(())
@@ -1712,8 +1845,6 @@ fn attach_console_devices(
 
     vmm.exit_observers.push(console.clone());
 
-    console.lock().unwrap().set_intc(intc);
-
     event_manager
         .add_subscriber(console.clone())
         .map_err(RegisterEvent)?;
@@ -1723,33 +1854,21 @@ fn attach_console_devices(
         .map_err(RegisterFsSigwinch)?;
 
     // The device mutex mustn't be locked here otherwise it will deadlock.
-    attach_mmio_device(
-        vmm,
-        "hvc0".to_string(),
-        MmioTransport::new(vmm.guest_memory().clone(), console),
-    )
-    .map_err(RegisterFsDevice)?;
-
+    attach_mmio_device(vmm, "hvc0".to_string(), intc, console).map_err(RegisterConsoleDevice)?;
     Ok(())
 }
 
 #[cfg(feature = "net")]
-fn attach_net_devices<'a>(
+fn attach_net_devices(
     vmm: &mut Vmm,
-    net_devices: impl Iterator<Item = &'a Arc<Mutex<Net>>>,
+    net_devices: &NetBuilder,
     intc: IrqChip,
 ) -> Result<(), StartMicrovmError> {
-    for net_device in net_devices {
+    for net_device in net_devices.list.iter() {
         let id = net_device.lock().unwrap().id().to_string();
 
-        net_device.lock().unwrap().set_intc(intc.clone());
-
-        attach_mmio_device(
-            vmm,
-            id,
-            MmioTransport::new(vmm.guest_memory.clone(), net_device.clone()),
-        )
-        .map_err(StartMicrovmError::RegisterNetDevice)?;
+        attach_mmio_device(vmm, id, intc.clone(), net_device.clone())
+            .map_err(StartMicrovmError::RegisterNetDevice)?;
     }
     Ok(())
 }
@@ -1768,15 +1887,8 @@ fn attach_unixsock_vsock_device(
 
     let id = String::from(unix_vsock.lock().unwrap().id());
 
-    unix_vsock.lock().unwrap().set_intc(intc);
-
     // The device mutex mustn't be locked here otherwise it will deadlock.
-    attach_mmio_device(
-        vmm,
-        id,
-        MmioTransport::new(vmm.guest_memory().clone(), unix_vsock.clone()),
-    )
-    .map_err(RegisterVsockDevice)?;
+    attach_mmio_device(vmm, id, intc, unix_vsock.clone()).map_err(RegisterVsockDevice)?;
 
     Ok(())
 }
@@ -1797,15 +1909,8 @@ fn attach_balloon_device(
 
     let id = String::from(balloon.lock().unwrap().id());
 
-    balloon.lock().unwrap().set_intc(intc);
-
     // The device mutex mustn't be locked here otherwise it will deadlock.
-    attach_mmio_device(
-        vmm,
-        id,
-        MmioTransport::new(vmm.guest_memory().clone(), balloon),
-    )
-    .map_err(RegisterBalloonDevice)?;
+    attach_mmio_device(vmm, id, intc.clone(), balloon).map_err(RegisterBalloonDevice)?;
 
     Ok(())
 }
@@ -1821,15 +1926,8 @@ fn attach_block_devices(
     for block in block_devs.list.iter() {
         let id = String::from(block.lock().unwrap().id());
 
-        block.lock().unwrap().set_intc(intc.clone());
-
         // The device mutex mustn't be locked here otherwise it will deadlock.
-        attach_mmio_device(
-            vmm,
-            id,
-            MmioTransport::new(vmm.guest_memory().clone(), block.clone()),
-        )
-        .map_err(RegisterBlockDevice)?;
+        attach_mmio_device(vmm, id, intc.clone(), block.clone()).map_err(RegisterBlockDevice)?;
     }
 
     Ok(())
@@ -1851,16 +1949,14 @@ fn attach_rng_device(
 
     let id = String::from(rng.lock().unwrap().id());
 
-    rng.lock().unwrap().set_intc(intc);
-
     // The device mutex mustn't be locked here otherwise it will deadlock.
-    attach_mmio_device(vmm, id, MmioTransport::new(vmm.guest_memory().clone(), rng))
-        .map_err(RegisterRngDevice)?;
+    attach_mmio_device(vmm, id, intc.clone(), rng).map_err(RegisterRngDevice)?;
 
     Ok(())
 }
 
 #[cfg(feature = "gpu")]
+#[allow(clippy::too_many_arguments)]
 fn attach_gpu_device(
     vmm: &mut Vmm,
     event_manager: &mut EventManager,
@@ -1868,6 +1964,8 @@ fn attach_gpu_device(
     #[cfg(not(feature = "tee"))] mut export_table: Option<ExportTable>,
     intc: IrqChip,
     virgl_flags: u32,
+    displays: Box<[DisplayInfo]>,
+    display_backend: DisplayBackend<'static>,
     #[cfg(target_os = "macos")] map_sender: Sender<WorkerMessage>,
 ) -> std::result::Result<(), StartMicrovmError> {
     use self::StartMicrovmError::*;
@@ -1875,6 +1973,8 @@ fn attach_gpu_device(
     let gpu = Arc::new(Mutex::new(
         devices::virtio::Gpu::new(
             virgl_flags,
+            displays,
+            display_backend,
             #[cfg(target_os = "macos")]
             map_sender,
         )
@@ -1887,8 +1987,6 @@ fn attach_gpu_device(
 
     let id = String::from(gpu.lock().unwrap().id());
 
-    gpu.lock().unwrap().set_intc(intc);
-
     if let Some(shm_region) = shm_manager.gpu_region() {
         gpu.lock().unwrap().set_shm_region(VirtioShmRegion {
             host_addr: vmm
@@ -1906,8 +2004,7 @@ fn attach_gpu_device(
     }
 
     // The device mutex mustn't be locked here otherwise it will deadlock.
-    attach_mmio_device(vmm, id, MmioTransport::new(vmm.guest_memory().clone(), gpu))
-        .map_err(RegisterGpuDevice)?;
+    attach_mmio_device(vmm, id, intc, gpu).map_err(RegisterGpuDevice)?;
 
     Ok(())
 }
@@ -1919,11 +2016,8 @@ fn attach_snd_device(vmm: &mut Vmm, intc: IrqChip) -> std::result::Result<(), St
     let snd = Arc::new(Mutex::new(devices::virtio::Snd::new().unwrap()));
     let id = String::from(snd.lock().unwrap().id());
 
-    snd.lock().unwrap().set_intc(intc);
-
     // The device mutex mustn't be locked here otherwise it will deadlock.
-    attach_mmio_device(vmm, id, MmioTransport::new(vmm.guest_memory().clone(), snd))
-        .map_err(RegisterSndDevice)?;
+    attach_mmio_device(vmm, id, intc, snd).map_err(RegisterSndDevice)?;
 
     Ok(())
 }
@@ -2012,62 +2106,62 @@ pub mod tests {
     fn test_error_messages() {
         use crate::builder::StartMicrovmError::*;
         let err = AttachBlockDevice(io::Error::from_raw_os_error(0));
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
 
         let err = CreateRateLimiter(io::Error::from_raw_os_error(0));
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
 
         let err = Internal(Error::Serial(io::Error::from_raw_os_error(0)));
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
 
         let err = InvalidKernelBundle(vm_memory::mmap::MmapRegionError::InvalidPointer);
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
 
         let err = KernelCmdline(String::from("dummy --cmdline"));
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
 
         let err = LoadCommandline(kernel::cmdline::Error::TooLarge);
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
 
         let err = MicroVMAlreadyRunning;
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
 
         let err = MissingKernelConfig;
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
 
         let err = MissingMemSizeConfig;
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
 
         let err = NetDeviceNotConfigured;
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
 
         let err = OpenBlockDevice(io::Error::from_raw_os_error(0));
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
 
         let err = RegisterBlockDevice(device_manager::mmio::Error::EventFd(
             io::Error::from_raw_os_error(0),
         ));
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
 
         let err = RegisterEvent(EventManagerError::EpollCreate(
             io::Error::from_raw_os_error(0),
         ));
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
 
         let err = RegisterNetDevice(device_manager::mmio::Error::EventFd(
             io::Error::from_raw_os_error(0),
         ));
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
 
         let err = RegisterVsockDevice(device_manager::mmio::Error::EventFd(
             io::Error::from_raw_os_error(0),
         ));
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
     }
 
     #[test]
     fn test_kernel_cmdline_err_to_startuvm_err() {
         let err = StartMicrovmError::from(kernel::cmdline::Error::HasSpace);
-        let _ = format!("{}{:?}", err, err);
+        let _ = format!("{err}{err:?}");
     }
 }
