@@ -23,6 +23,8 @@ use crate::vstate::Vm;
 #[allow(clippy::enum_variant_names)]
 #[derive(Debug)]
 pub enum Error {
+    /// Failed to create MmioTransport
+    CreateMmioTransport(devices::virtio::CreateMmioTransportError),
     /// Failed to perform an operation on the bus.
     BusError(devices::BusError),
     /// Appending to kernel command line failed.
@@ -44,11 +46,14 @@ pub enum Error {
 impl fmt::Display for Error {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
-            Error::BusError(ref e) => write!(f, "failed to perform bus operation: {}", e),
-            Error::Cmdline(ref e) => {
-                write!(f, "unable to add device to kernel command line: {}", e)
+            Error::CreateMmioTransport(ref e) => {
+                write!(f, "failed to create mmio transport for the device {e}")
             }
-            Error::EventFd(ref e) => write!(f, "failed to create or clone event descriptor: {}", e),
+            Error::BusError(ref e) => write!(f, "failed to perform bus operation: {e}"),
+            Error::Cmdline(ref e) => {
+                write!(f, "unable to add device to kernel command line: {e}")
+            }
+            Error::EventFd(ref e) => write!(f, "failed to create or clone event descriptor: {e}"),
             Error::IrqsExhausted => write!(f, "no more IRQs are available"),
             Error::RegisterIoEvent => write!(f, "failed to register IO event"),
             Error::RegisterIrqFd => write!(f, "failed to register irqfd"),
@@ -58,6 +63,12 @@ impl fmt::Display for Error {
     }
 }
 
+impl From<devices::virtio::CreateMmioTransportError> for crate::device_manager::mmio::Error {
+    fn from(e: devices::virtio::CreateMmioTransportError) -> Self {
+        Self::CreateMmioTransport(e)
+    }
+}
+
 type Result<T> = ::std::result::Result<T, Error>;
 
 /// This represents the size of the mmio device specified to the kernel as a cmdline option
@@ -112,7 +123,7 @@ impl MMIODeviceManager {
             mmio_device.register_queue_evt(queue_evt, i as u32);
         }
 
-        mmio_device.locked_device().set_irq_line(self.irq);
+        mmio_device.set_irq_line(self.irq);
 
         self.bus
             .insert(Arc::new(Mutex::new(mmio_device)), self.mmio_base, MMIO_LEN)
@@ -326,7 +337,8 @@ mod tests {
     use super::super::super::builder;
     use super::*;
     use arch;
-    use devices::virtio::{ActivateResult, Queue, VirtioDevice};
+    use devices::legacy::DummyIrqChip;
+    use devices::virtio::{ActivateResult, InterruptTransport, Queue, VirtioDevice};
     use std::sync::atomic::AtomicUsize;
     use std::sync::Arc;
     use utils::errno;
@@ -345,7 +357,8 @@ mod tests {
             type_id: u32,
             device_id: &str,
         ) -> Result<u64> {
-            let mmio_device = devices::virtio::MmioTransport::new(guest_mem, device);
+            let mmio_device =
+                devices::virtio::MmioTransport::new(guest_mem, DummyIrqChip::new().into(), device);
             let (mmio_base, _irq) =
                 self.register_mmio_device(vm, mmio_device, type_id, device_id.to_string())?;
             #[cfg(target_arch = "x86_64")]
@@ -403,14 +416,6 @@ mod tests {
             &self.queue_evts
         }
 
-        fn interrupt_evt(&self) -> &EventFd {
-            &self.interrupt_evt
-        }
-
-        fn interrupt_status(&self) -> Arc<AtomicUsize> {
-            Arc::new(AtomicUsize::new(0))
-        }
-
         fn ack_features_by_page(&mut self, page: u32, value: u32) {
             let _ = page;
             let _ = value;
@@ -426,7 +431,11 @@ mod tests {
             let _ = data;
         }
 
-        fn activate(&mut self, _: GuestMemoryMmap) -> ActivateResult {
+        fn activate(
+            &mut self,
+            mem: GuestMemoryMmap,
+            _interrupt: InterruptTransport,
+        ) -> ActivateResult {
             Ok(())
         }
 
