@@ -9,7 +9,7 @@ use std::collections::HashMap;
 use std::sync::{Arc, Mutex};
 use std::{fmt, io};
 
-#[cfg(target_arch = "aarch64")]
+#[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
 use devices::fdt::DeviceInfoForFDT;
 use devices::{BusDevice, DeviceType};
 use kernel::cmdline as kernel_cmdline;
@@ -21,6 +21,8 @@ use utils::eventfd::EventFd;
 #[allow(clippy::enum_variant_names)]
 #[derive(Debug)]
 pub enum Error {
+    /// Failed to create MmioTransport
+    CreateMmioTransport(devices::virtio::CreateMmioTransportError),
     /// Failed to perform an operation on the bus.
     BusError(devices::BusError),
     /// Appending to kernel command line failed.
@@ -42,6 +44,9 @@ pub enum Error {
 impl fmt::Display for Error {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
+            Error::CreateMmioTransport(ref e) => {
+                write!(f, "failed to create mmio transport for the device {e}")
+            }
             Error::BusError(ref e) => write!(f, "failed to perform bus operation: {e}"),
             Error::Cmdline(ref e) => {
                 write!(f, "unable to add device to kernel command line: {e}")
@@ -56,6 +61,12 @@ impl fmt::Display for Error {
     }
 }
 
+impl From<devices::virtio::CreateMmioTransportError> for Error {
+    fn from(e: devices::virtio::CreateMmioTransportError) -> Self {
+        Self::CreateMmioTransport(e)
+    }
+}
+
 type Result<T> = ::std::result::Result<T, Error>;
 
 /// This represents the size of the mmio device specified to the kernel as a cmdline option
@@ -76,7 +87,7 @@ pub struct MMIODeviceManager {
 impl MMIODeviceManager {
     /// Create a new DeviceManager handling mmio devices (virtio net, block).
     pub fn new(mmio_base: &mut u64, irq_interval: (u32, u32)) -> MMIODeviceManager {
-        if cfg!(target_arch = "aarch64") {
+        if cfg!(any(target_arch = "aarch64", target_arch = "riscv64")) {
             *mmio_base += MMIO_LEN;
         }
         MMIODeviceManager {
@@ -109,7 +120,7 @@ impl MMIODeviceManager {
     pub fn register_mmio_device(
         &mut self,
         vm: &VmFd,
-        mmio_device: devices::virtio::MmioTransport,
+        mut mmio_device: devices::virtio::MmioTransport,
         type_id: u32,
         device_id: String,
     ) -> Result<(u64, u32)> {
@@ -131,10 +142,10 @@ impl MMIODeviceManager {
                 .map_err(Error::RegisterIoEvent)?;
         }
 
-        vm.register_irqfd(mmio_device.locked_device().interrupt_evt(), self.irq)
+        vm.register_irqfd(mmio_device.interrupt_evt(), self.irq)
             .map_err(Error::RegisterIrqFd)?;
 
-        mmio_device.locked_device().set_irq_line(self.irq);
+        mmio_device.set_irq_line(self.irq);
 
         self.bus
             .insert(Arc::new(Mutex::new(mmio_device)), self.mmio_base, MMIO_LEN)
@@ -175,7 +186,7 @@ impl MMIODeviceManager {
             .map_err(Error::Cmdline)
     }
 
-    #[cfg(target_arch = "aarch64")]
+    #[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
     /// Register an early console at some MMIO address.
     pub fn register_mmio_serial(
         &mut self,
@@ -197,7 +208,10 @@ impl MMIODeviceManager {
         cmdline
             .insert(
                 "earlycon",
+                #[cfg(target_arch = "aarch64")]
                 &format!("pl011,mmio32,0x{:08x}", self.mmio_base),
+                #[cfg(target_arch = "riscv64")]
+                &format!("uart,mmio,0x{:08x}", self.mmio_base),
             )
             .map_err(Error::Cmdline)?;
 
@@ -250,7 +264,7 @@ impl MMIODeviceManager {
         Ok(())
     }
 
-    #[cfg(target_arch = "aarch64")]
+    #[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
     /// Gets the information of the devices registered up to some point in time.
     pub fn get_device_info(&self) -> &HashMap<(DeviceType, String), MMIODeviceInfo> {
         &self.id_to_dev_info
@@ -282,7 +296,7 @@ pub struct MMIODeviceInfo {
     _len: u64,
 }
 
-#[cfg(target_arch = "aarch64")]
+#[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
 impl DeviceInfoForFDT for MMIODeviceInfo {
     fn addr(&self) -> u64 {
         self.addr
@@ -300,11 +314,12 @@ mod tests {
     use super::super::super::super::builder;
     use super::*;
     use arch;
+    use devices::legacy::DummyIrqChip;
+    use devices::virtio::InterruptTransport;
     use devices::{
         legacy::KvmIoapic,
         virtio::{ActivateResult, Queue, VirtioDevice},
     };
-    use std::sync::atomic::AtomicUsize;
     use std::sync::Arc;
     use utils::errno;
     use utils::eventfd::EventFd;
@@ -322,7 +337,9 @@ mod tests {
             type_id: u32,
             device_id: &str,
         ) -> Result<u64> {
-            let mmio_device = devices::virtio::MmioTransport::new(guest_mem, device);
+            let mmio_device =
+                devices::virtio::MmioTransport::new(guest_mem, DummyIrqChip::new().into(), device)
+                    .unwrap();
             let (mmio_base, _irq) =
                 self.register_mmio_device(vm, mmio_device, type_id, device_id.to_string())?;
             #[cfg(target_arch = "x86_64")]
@@ -368,6 +385,10 @@ mod tests {
             0
         }
 
+        fn device_name(&self) -> &str {
+            "dummy"
+        }
+
         fn queues(&self) -> &[Queue] {
             &self.queues
         }
@@ -380,16 +401,6 @@ mod tests {
             &self.queue_evts
         }
 
-        fn interrupt_evt(&self) -> &EventFd {
-            &self.interrupt_evt
-        }
-
-        fn interrupt_status(&self) -> Arc<AtomicUsize> {
-            Arc::new(AtomicUsize::new(0))
-        }
-
-        fn set_irq_line(&mut self, _irq: u32) {}
-
         fn ack_features_by_page(&mut self, page: u32, value: u32) {
             let _ = page;
             let _ = value;
@@ -405,7 +416,7 @@ mod tests {
             let _ = data;
         }
 
-        fn activate(&mut self, _: GuestMemoryMmap) -> ActivateResult {
+        fn activate(&mut self, _mem: GuestMemoryMmap, _intc: InterruptTransport) -> ActivateResult {
             Ok(())
         }
 
@@ -504,7 +515,7 @@ mod tests {
                 .unwrap_err(),
         );
         assert_eq!(
-            format!("{}", e),
+            format!("{e}"),
             format!(
                 "unable to add device to kernel command line: {}",
                 kernel_cmdline::Error::HasEquals
