@@ -14,17 +14,24 @@ use std::ops::Range;
 
 use std::os::unix::io::RawFd;
 
+#[cfg(target_arch = "x86_64")]
+use std::env;
 use std::result;
 use std::sync::atomic::{fence, Ordering};
 #[cfg(not(test))]
 use std::sync::Barrier;
 use std::thread;
+#[cfg(target_arch = "x86_64")]
+use std::time::Duration;
 
 use super::super::{FC_EXIT_CODE_GENERIC_ERROR, FC_EXIT_CODE_OK};
 
 #[cfg(feature = "amd-sev")]
 use super::tee::amdsnp::{AmdSnp, Error as SnpError};
 
+#[cfg(feature = "tdx")]
+use super::tee::inteltdx::{Error as TdxError, IntelTdx};
+
 #[cfg(feature = "tee")]
 use kbs_types::Tee;
 
@@ -41,12 +48,13 @@ use kvm_bindings::{
     KVM_MAX_CPUID_ENTRIES,
 };
 use kvm_bindings::{
-    kvm_create_guest_memfd, kvm_memory_attributes, kvm_userspace_memory_region,
-    kvm_userspace_memory_region2, KVM_API_VERSION, KVM_MEMORY_ATTRIBUTE_PRIVATE,
-    KVM_MEM_GUEST_MEMFD, KVM_SYSTEM_EVENT_RESET, KVM_SYSTEM_EVENT_SHUTDOWN,
+    kvm_create_guest_memfd, kvm_userspace_memory_region, kvm_userspace_memory_region2,
+    KVM_API_VERSION, KVM_MEM_GUEST_MEMFD, KVM_SYSTEM_EVENT_RESET, KVM_SYSTEM_EVENT_SHUTDOWN,
 };
 #[cfg(feature = "tee")]
 use kvm_bindings::{kvm_enable_cap, KVM_CAP_EXIT_HYPERCALL, KVM_MEMORY_EXIT_FLAG_PRIVATE};
+#[cfg(not(target_arch = "riscv64"))]
+use kvm_bindings::{kvm_memory_attributes, KVM_MEMORY_ATTRIBUTE_PRIVATE};
 use kvm_ioctls::{Cap::*, *};
 use utils::eventfd::EventFd;
 use utils::signal::{register_signal_handler, sigrtmin, Killable};
@@ -108,6 +116,9 @@ pub enum Error {
     #[cfg(target_arch = "aarch64")]
     /// Error configuring the general purpose aarch64 registers.
     REGSConfiguration(arch::aarch64::regs::Error),
+    #[cfg(target_arch = "riscv64")]
+    /// Error configuring the general purpose riscv64 registers.
+    REGSConfiguration(arch::riscv64::regs::Error),
     #[cfg(target_arch = "x86_64")]
     /// Error configuring the general purpose registers
     REGSConfiguration(arch::x86_64::regs::Error),
@@ -126,6 +137,12 @@ pub enum Error {
     #[cfg(feature = "amd-sev")]
     /// Error attesting the Secure VM (SNP).
     SnpSecVirtAttest(SnpError),
+    #[cfg(feature = "tdx")]
+    /// Error preparing the VM for Trust Domain Extensions (TDX)
+    TdxSecVirtPrepare(TdxError),
+    #[cfg(feature = "tdx")]
+    /// Error initializing vCPU for Trust Domain Extensions (TDX)
+    TdxSecVirtInitVcpu,
     #[cfg(feature = "tee")]
     /// The TEE specified is not supported.
     InvalidTee,
@@ -236,6 +253,10 @@ pub enum Error {
     VmSetIrqChip(kvm_ioctls::Error),
     /// Cannot configure the microvm.
     VmSetup(kvm_ioctls::Error),
+    /// Failed to enable split IRQCHIP in vm
+    VmSplitIrqchip(kvm_ioctls::Error),
+    /// Failed to set vm APIC bus clock rate (in nanoseconds)
+    VmApicBusClockRate(kvm_ioctls::Error),
 }
 
 impl Display for Error {
@@ -262,6 +283,11 @@ impl Display for Error {
             VmFd(e) => write!(f, "Cannot open the VM file descriptor: {e}"),
             VcpuFd(e) => write!(f, "Cannot open the VCPU file descriptor: {e}"),
             VmSetup(e) => write!(f, "Cannot configure the microvm: {e}"),
+            VmSplitIrqchip(e) => write!(f, "Failed to enable split IRQCHIP: {e}"),
+            VmApicBusClockRate(e) => write!(
+                f,
+                "Failed to set vm APIC bus clock rate (in nanoseconds): {e}"
+            ),
             VcpuRun(e) => write!(f, "Cannot run the VCPUs: {e}"),
             NotEnoughMemorySlots => write!(
                 f,
@@ -275,22 +301,32 @@ impl Display for Error {
             SetMemoryAttributes(e) => write!(f, "Cannot set memory region attributes: {e}"),
             SetUserMemoryRegion(e) => write!(f, "Cannot set the memory regions: {e}"),
             ShmMmap(e) => write!(f, "Error creating memory map for SHM region: {e}"),
-            #[cfg(feature = "tee")]
+            #[cfg(feature = "amd-sev")]
             SnpSecVirtInit(e) => write!(
                 f,
                 "Error initializing the Secure Virtualization Backend (SEV): {e:?}"
             ),
 
-            #[cfg(feature = "tee")]
+            #[cfg(feature = "amd-sev")]
             SnpSecVirtPrepare(e) => write!(
                 f,
                 "Error preparing the VM for Secure Virtualization (SNP): {e:?}"
             ),
 
-            #[cfg(feature = "tee")]
+            #[cfg(feature = "amd-sev")]
             SnpSecVirtAttest(e) => write!(f, "Error attesting the Secure VM (SNP): {e:?}"),
 
             SignalVcpu(e) => write!(f, "Failed to signal Vcpu: {e}"),
+            #[cfg(feature = "tdx")]
+            TdxSecVirtPrepare(e) => write!(
+                f,
+                "Error preparing the VM for Trust Domain Extensions (TDX): {e:?}"
+            ),
+            #[cfg(feature = "tdx")]
+            TdxSecVirtInitVcpu => write!(
+                f,
+                "Error initializing vCPU for Trust Domain Extensions (TDX)"
+            ),
             #[cfg(feature = "tee")]
             MissingTeeConfig => write!(f, "Missing TEE configuration"),
             #[cfg(target_arch = "x86_64")]
@@ -300,6 +336,11 @@ impl Display for Error {
                 f,
                 "Error configuring the general purpose aarch64 registers: {e:?}"
             ),
+            #[cfg(target_arch = "riscv64")]
+            REGSConfiguration(e) => write!(
+                f,
+                "Error configuring the general purpose riscv64 registers: {e:?}"
+            ),
             #[cfg(target_arch = "x86_64")]
             REGSConfiguration(e) => {
                 write!(f, "Error configuring the general purpose registers: {e:?}")
@@ -415,6 +456,9 @@ impl KvmContext {
         #[cfg(target_arch = "aarch64")]
         let capabilities = [Irqchip, Ioeventfd, Irqfd, UserMemory, ArmPsci02];
 
+        #[cfg(target_arch = "riscv64")]
+        let capabilities = [Irqchip, Ioeventfd, Irqfd, UserMemory];
+
         // Check that all desired capabilities are supported.
         match capabilities
             .iter()
@@ -453,7 +497,10 @@ pub struct Vm {
     #[cfg(feature = "amd-sev")]
     tee: Option<AmdSnp>,
 
-    #[cfg(feature = "amd-sev")]
+    #[cfg(feature = "tdx")]
+    tdx: Option<IntelTdx>,
+
+    #[cfg(feature = "tee")]
     pub tee_config: Tee,
 
     pub guest_memfds: Vec<(Range<u64>, RawFd)>,
@@ -524,6 +571,51 @@ impl Vm {
         })
     }
 
+    #[cfg(feature = "tdx")]
+    pub fn new(
+        kvm: &Kvm,
+        tee_config: &TeeConfig,
+        _sender: crossbeam_channel::Sender<WorkerMessage>,
+    ) -> Result<Self> {
+        // create fd for interacting with kvm-vm specific functions
+        let vm_fd = kvm
+            .create_vm_with_type(tdx::launch::KVM_X86_TDX_VM)
+            .map_err(Error::VmFd)?;
+
+        let supported_cpuid = kvm
+            .get_supported_cpuid(KVM_MAX_CPUID_ENTRIES)
+            .map_err(Error::VmFd)?;
+
+        let supported_msrs =
+            arch::x86_64::msr::supported_guest_msrs(kvm).map_err(Error::GuestMSRs)?;
+
+        let mut cap = kvm_enable_cap {
+            cap: KVM_CAP_EXIT_HYPERCALL,
+            flags: 0,
+            args: [1 << 12 /* KVM_HC_MAP_GPA_RANGE */, 0, 0, 0],
+            ..Default::default()
+        };
+        vm_fd.enable_cap(&cap).map_err(Error::HypercallExitEnable)?;
+
+        cap.cap = kvm_bindings::KVM_CAP_SPLIT_IRQCHIP;
+        cap.args[0] = 24;
+        vm_fd.enable_cap(&cap).map_err(Error::VmSplitIrqchip)?;
+
+        cap.cap = 237; // KVM_CAP_X86_APIC_BUS_CYCLES_NS
+        cap.args[0] = 40;
+        vm_fd.enable_cap(&cap).map_err(Error::VmApicBusClockRate)?;
+
+        Ok(Vm {
+            fd: vm_fd,
+            next_mem_slot: 0,
+            supported_cpuid,
+            supported_msrs,
+            tdx: Some(IntelTdx::new()),
+            tee_config: tee_config.tee,
+            guest_memfds: Vec::new(),
+        })
+    }
+
     /// Returns a ref to the supported `CpuId` for this Vm.
     #[cfg(any(target_arch = "x86", target_arch = "x86_64"))]
     pub fn supported_cpuid(&self) -> &CpuId {
@@ -624,6 +716,7 @@ impl Vm {
                     .map_err(Error::SetUserMemoryRegion)?;
             };
 
+            #[cfg(not(target_arch = "riscv64"))]
             let attr = kvm_memory_attributes {
                 address: start,
                 size: region.len(),
@@ -631,6 +724,7 @@ impl Vm {
                 flags: 0,
             };
 
+            #[cfg(not(target_arch = "riscv64"))]
             self.fd
                 .set_memory_attributes(attr)
                 .map_err(Error::SetMemoryAttributes)?;
@@ -643,6 +737,49 @@ impl Vm {
         Ok(())
     }
 
+    #[cfg(feature = "tdx")]
+    pub fn tdx_secure_virt_prepare(&self) -> Result<tdx::launch::Launcher> {
+        match &self.tdx {
+            Some(t) => t
+                .vm_prepare(&self.fd, self.supported_cpuid.clone())
+                .map_err(Error::TdxSecVirtPrepare),
+            None => Err(Error::InvalidTee),
+        }
+    }
+
+    #[cfg(feature = "tdx")]
+    pub fn tdx_secure_virt_init_vcpus(&self, launcher: &mut tdx::launch::Launcher) -> Result<()> {
+        match &self.tdx {
+            Some(_) => {
+                launcher.init_vcpus(0).unwrap();
+                Ok(())
+            }
+            None => Err(Error::InvalidTee),
+        }
+    }
+
+    #[cfg(feature = "tdx")]
+    pub fn tdx_secure_virt_prepare_memory(
+        &self,
+        launcher: &mut tdx::launch::Launcher,
+        regions: &Vec<crate::vstate::MeasuredRegion>,
+    ) -> Result<()> {
+        match &self.tdx {
+            Some(t) => t
+                .configure_td_memory(launcher, regions)
+                .map_err(Error::TdxSecVirtPrepare),
+            None => Err(Error::InvalidTee),
+        }
+    }
+
+    #[cfg(feature = "tdx")]
+    pub fn tdx_secure_virt_finalize_vm(&self, launcher: tdx::launch::Launcher) -> Result<()> {
+        match &self.tdx {
+            Some(t) => t.finalize_vm(launcher).map_err(Error::TdxSecVirtPrepare),
+            None => Err(Error::InvalidTee),
+        }
+    }
+
     #[cfg(feature = "amd-sev")]
     pub fn snp_secure_virt_prepare(
         &self,
@@ -781,6 +918,8 @@ pub struct Vcpu {
     cpuid: CpuId,
     #[cfg(target_arch = "x86_64")]
     msr_list: MsrList,
+    #[cfg(target_arch = "x86_64")]
+    kernel_enomem_workaround: bool,
 
     #[cfg(target_arch = "aarch64")]
     mpidr: u64,
@@ -905,6 +1044,13 @@ impl Vcpu {
         let (event_sender, event_receiver) = unbounded();
         let (response_sender, response_receiver) = unbounded();
 
+        let kernel_enomem_workaround = if env::var_os("KRUN_ENOMEM_WORKAROUND").is_some() {
+            debug!("Enabling ENOMEM workaround");
+            true
+        } else {
+            false
+        };
+
         // Initially the cpuid per vCPU is the one supported by this VM.
         Ok(Vcpu {
             fd: kvm_vcpu,
@@ -914,6 +1060,7 @@ impl Vcpu {
             io_bus,
             cpuid,
             msr_list,
+            kernel_enomem_workaround,
             event_receiver,
             event_sender: Some(event_sender),
             response_receiver: Some(response_receiver),
@@ -950,6 +1097,32 @@ impl Vcpu {
         })
     }
 
+    /// Constructs a new VCPU for `vm`.
+    ///
+    /// # Arguments
+    ///
+    /// * `id` - Represents the CPU number between [0, max vcpus).
+    /// * `vm_fd` - The kvm `VmFd` for the virtual machine this vcpu will get attached to.
+    /// * `exit_evt` - An `EventFd` that will be written into when this vcpu exits.
+    /// * `create_ts` - A timestamp used by the vcpu to calculate its lifetime.
+    #[cfg(target_arch = "riscv64")]
+    pub fn new_riscv64(id: u8, vm_fd: &VmFd, exit_evt: EventFd) -> Result<Self> {
+        let kvm_vcpu = vm_fd.create_vcpu(id as u64).map_err(Error::VcpuFd)?;
+        let (event_sender, event_receiver) = unbounded();
+        let (response_sender, response_receiver) = unbounded();
+
+        Ok(Vcpu {
+            fd: kvm_vcpu,
+            id,
+            mmio_bus: None,
+            exit_evt,
+            event_receiver,
+            event_sender: Some(event_sender),
+            response_receiver: Some(response_receiver),
+            response_sender,
+        })
+    }
+
     /// Returns the cpu index as seen by the guest OS.
     pub fn cpu_index(&self) -> u8 {
         self.id
@@ -1004,13 +1177,16 @@ impl Vcpu {
             .set_cpuid2(&self.cpuid)
             .map_err(Error::VcpuSetCpuid)?;
 
-        arch::x86_64::msr::setup_msrs(&self.fd).map_err(Error::MSRSConfiguration)?;
-        arch::x86_64::regs::setup_regs(&self.fd, kernel_start_addr.raw_value(), self.id)
-            .map_err(Error::REGSConfiguration)?;
-        arch::x86_64::regs::setup_fpu(&self.fd).map_err(Error::FPUConfiguration)?;
-        arch::x86_64::regs::setup_sregs(guest_mem, &self.fd, self.id)
-            .map_err(Error::SREGSConfiguration)?;
-        arch::x86_64::interrupts::set_lint(&self.fd).map_err(Error::LocalIntConfiguration)?;
+        #[cfg(not(feature = "tdx"))]
+        {
+            arch::x86_64::msr::setup_msrs(&self.fd).map_err(Error::MSRSConfiguration)?;
+            arch::x86_64::regs::setup_regs(&self.fd, kernel_start_addr.raw_value(), self.id)
+                .map_err(Error::REGSConfiguration)?;
+            arch::x86_64::regs::setup_fpu(&self.fd).map_err(Error::FPUConfiguration)?;
+            arch::x86_64::regs::setup_sregs(guest_mem, &self.fd, self.id)
+                .map_err(Error::SREGSConfiguration)?;
+            arch::x86_64::interrupts::set_lint(&self.fd).map_err(Error::LocalIntConfiguration)?;
+        }
         Ok(())
     }
 
@@ -1041,6 +1217,13 @@ impl Vcpu {
             kvi.features[0] |= 1 << kvm_bindings::KVM_ARM_VCPU_POWER_OFF;
         }
 
+        if vm_fd.check_extension(kvm_ioctls::Cap::ArmPtrAuthAddress) {
+            kvi.features[0] |= 1 << kvm_bindings::KVM_ARM_VCPU_PTRAUTH_ADDRESS;
+        }
+        if vm_fd.check_extension(kvm_ioctls::Cap::ArmPtrAuthGeneric) {
+            kvi.features[0] |= 1 << kvm_bindings::KVM_ARM_VCPU_PTRAUTH_GENERIC;
+        }
+
         self.fd.vcpu_init(&kvi).map_err(Error::VcpuArmInit)?;
         arch::aarch64::regs::setup_regs(&self.fd, self.id, kernel_load_addr.raw_value(), guest_mem)
             .map_err(Error::REGSConfiguration)?;
@@ -1050,6 +1233,25 @@ impl Vcpu {
         Ok(())
     }
 
+    #[cfg(target_arch = "riscv64")]
+    /// Configures an riscv64 specific vcpu.
+    ///
+    /// # Arguments
+    ///
+    /// * `vm_fd` - The kvm `VmFd` for this microvm.
+    /// * `guest_mem` - The guest memory used by this microvm.
+    /// * `kernel_load_addr` - Offset from `guest_mem` at which the kernel is loaded.
+    pub fn configure_riscv64(
+        &mut self,
+        _vm_fd: &VmFd,
+        guest_mem: &GuestMemoryMmap,
+        kernel_load_addr: GuestAddress,
+    ) -> Result<()> {
+        arch::riscv64::regs::setup_regs(&self.fd, self.id, kernel_load_addr.raw_value(), guest_mem)
+            .map_err(Error::REGSConfiguration)?;
+        Ok(())
+    }
+
     /// Moves the vcpu to its own thread and constructs a VcpuHandle.
     /// The handle can be used to control the remote vcpu.
     pub fn start_threaded(mut self) -> Result<VcpuHandle> {
@@ -1203,6 +1405,16 @@ impl Vcpu {
     ///
     /// Returns error or enum specifying whether emulation was handled or interrupted.
     fn run_emulation(&mut self) -> Result<VcpuEmulation> {
+        // This is a workaround for a kernel bug in the Linux
+        // kernel (6.12 and 6.13).
+        // https://github.com/containers/libkrun/issues/314#issuecomment-2818154193
+        #[cfg(target_arch = "x86_64")]
+        {
+            if self.kernel_enomem_workaround {
+                thread::sleep(Duration::from_millis(5));
+            }
+        }
+
         match self.fd.run() {
             Ok(run) => match run {
                 #[cfg(feature = "tee")]
@@ -1229,7 +1441,7 @@ impl Vcpu {
                         ))
                         .unwrap();
                     if !response_receiver.recv().unwrap() {
-                        error!("Unable to convert memory with properties: gpa: 0x{:x} size: 0x{:x} to_private: {}", gpa, size, private);
+                        error!("Unable to convert memory with properties: gpa: 0x{gpa:x} size: 0x{size:x} to_private: {private}");
                         return Err(Error::VcpuUnhandledKvmExit);
                     }
                     Ok(VcpuEmulation::Handled)
@@ -1246,23 +1458,25 @@ impl Vcpu {
                 }
                 #[cfg(feature = "tee")]
                 VcpuExit::MemoryFault { gpa, size, flags } => {
-                    let private = (flags & (KVM_MEMORY_EXIT_FLAG_PRIVATE as u64)) != 0;
-
-                    let mem_properties = MemoryProperties { gpa, size, private };
-
-                    let (response_sender, response_receiver) = unbounded();
-                    self.pm_sender
-                        .send(WorkerMessage::ConvertMemory(
-                            response_sender.clone(),
-                            mem_properties,
-                        ))
-                        .unwrap();
-                    if !response_receiver.recv().unwrap() {
-                        error!("Unable to convert memory with properties: gpa: 0x{:x} size: 0x{:x} to_private: {}", gpa, size, private);
-                        return Err(Error::VcpuUnhandledKvmExit);
+                    if flags & !kvm_bindings::KVM_MEMORY_EXIT_FLAG_PRIVATE as u64 != 0 {
+                        println!("KVM_EXIT_MEMORY_FAULT: Unknown flag {flags}");
+                        Err(Error::VcpuUnhandledKvmExit)
+                    } else {
+                        let private = (flags & (KVM_MEMORY_EXIT_FLAG_PRIVATE as u64)) != 0;
+                        let mem_properties = MemoryProperties { gpa, size, private };
+                        let (response_sender, response_receiver) = unbounded();
+                        self.pm_sender
+                            .send(WorkerMessage::ConvertMemory(
+                                response_sender.clone(),
+                                mem_properties,
+                            ))
+                            .unwrap();
+                        if !response_receiver.recv().unwrap() {
+                            error!("Unable to convert memory with properties: gpa: 0x{gpa:x} size: 0x{size:x} to_private: {private}");
+                            return Err(Error::VcpuUnhandledKvmExit);
+                        }
+                        Ok(VcpuEmulation::Handled)
                     }
-
-                    Ok(VcpuEmulation::Handled)
                 }
                 VcpuExit::MmioRead(addr, data) => {
                     if let Some(ref mmio_bus) = self.mmio_bus {
@@ -1305,7 +1519,7 @@ impl Vcpu {
                 r => {
                     // TODO: Are we sure we want to finish running a vcpu upon
                     // receiving a vm exit that is not necessarily an error?
-                    error!("Unexpected exit reason on vcpu run: {:?}", r);
+                    error!("Unexpected exit reason on vcpu run: {r:?}");
                     Err(Error::VcpuUnhandledKvmExit)
                 }
             },
@@ -1320,7 +1534,7 @@ impl Vcpu {
                         Ok(VcpuEmulation::Interrupted)
                     }
                     _ => {
-                        error!("Failure during vcpu run: {}", e);
+                        error!("Failure during vcpu run: {e}");
                         Err(Error::VcpuUnhandledKvmExit)
                     }
                 }
@@ -1426,7 +1640,7 @@ impl Vcpu {
             .expect("failed to send Exited status");
 
         if let Err(e) = self.exit_evt.write(1) {
-            error!("Failed signaling vcpu exit event: {}", e);
+            error!("Failed signaling vcpu exit event: {e}");
         }
 
         // State machine reached its end.
@@ -1444,6 +1658,12 @@ impl Vcpu {
         StateMachine::finish()
     }
 
+    #[cfg(feature = "tdx")]
+    pub fn tdx_secure_virt_prepare(&self, launcher: &mut tdx::launch::Launcher) {
+        use std::os::fd::AsRawFd;
+        launcher.add_vcpu_fd(self.fd.as_raw_fd());
+    }
+
     #[cfg(test)]
     // In tests the main/vmm thread exits without 'exit()'ing the whole process.
     // All channels get closed on the other side while this Vcpu thread is still running.
