@@ -12,9 +12,6 @@ use std::path::PathBuf;
 #[cfg(feature = "tee")]
 use serde::{Deserialize, Serialize};
 
-#[cfg(feature = "tee")]
-use kbs_types::Tee;
-
 #[cfg(feature = "blk")]
 use crate::vmm_config::block::{BlockBuilder, BlockConfigError, BlockDeviceConfig};
 use crate::vmm_config::boot_source::{BootSourceConfig, BootSourceConfigError};
@@ -30,6 +27,13 @@ use crate::vmm_config::net::{NetBuilder, NetworkInterfaceConfig, NetworkInterfac
 use crate::vmm_config::vsock::*;
 use crate::vstate::VcpuConfig;
 
+#[cfg(feature = "gpu")]
+use devices::virtio::display::DisplayInfo;
+#[cfg(feature = "tee")]
+use kbs_types::Tee;
+#[cfg(feature = "gpu")]
+use krun_display::DisplayBackend;
+
 type Result<E> = std::result::Result<(), E>;
 
 /// Errors encountered when configuring microVM resources.
@@ -104,13 +108,17 @@ pub struct VmResources {
     pub block: BlockBuilder,
     /// The network devices builder.
     #[cfg(feature = "net")]
-    pub net_builder: NetBuilder,
+    pub net: NetBuilder,
     /// TEE configuration
     #[cfg(feature = "tee")]
     pub tee_config: TeeConfig,
     /// Flags for the virtio-gpu device.
     pub gpu_virgl_flags: Option<u32>,
     pub gpu_shm_size: Option<usize>,
+    #[cfg(feature = "gpu")]
+    pub display_backend: Option<DisplayBackend<'static>>,
+    #[cfg(feature = "gpu")]
+    pub displays: Vec<DisplayInfo>,
     #[cfg(feature = "snd")]
     /// Enable the virtio-snd device.
     pub snd_device: bool,
@@ -282,8 +290,7 @@ impl VmResources {
         &mut self,
         config: NetworkInterfaceConfig,
     ) -> Result<NetworkInterfaceError> {
-        self.net_builder.build(config)?;
-        Ok(())
+        self.net.insert(config)
     }
 
     #[cfg(feature = "tee")]
@@ -315,6 +322,8 @@ impl VmResources {
 
 #[cfg(test)]
 mod tests {
+    #[cfg(feature = "gpu")]
+    use crate::resources::DisplayBackendConfig;
     use crate::resources::VmResources;
     use crate::vmm_config::boot_source::BootSourceConfig;
     use crate::vmm_config::machine_config::{CpuFeaturesTemplate, VmConfig, VmConfigError};
@@ -325,6 +334,7 @@ mod tests {
     fn default_boot_cfg() -> BootSourceConfig {
         BootSourceConfig {
             kernel_cmdline_prolog: None,
+            kernel_cmdline_krun_env: None,
             kernel_cmdline_epilog: None,
         }
     }
@@ -341,6 +351,10 @@ mod tests {
             net_builder: Default::default(),
             gpu_virgl_flags: None,
             gpu_shm_size: None,
+            #[cfg(feature = "gpu")]
+            display_backend: DisplayBackendConfig::Noop,
+            #[cfg(feature = "gpu")]
+            displays: Vec::new(),
             #[cfg(feature = "snd")]
             enable_snd: False,
             console_output: None,
