@@ -8,6 +8,9 @@ pub mod block;
 /// Wrapper for configuring the microVM boot source.
 pub mod boot_source;
 
+/// Wrapper for configuring an external kernel to be loaded in the microVM.
+pub mod external_kernel;
+
 /// Wrapper for configuring the Fs devices attached to the microVM.
 #[cfg(not(feature = "tee"))]
 pub mod fs;
